def iterate(client=None):
    r"""
    Makes iteration agent.

    ..  container:: example

        Iterates leaves:

        >>> staff = abjad.Staff("c'4 e'4 d'4 f'4")
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                e'4
                d'4
                f'4
            }

        >>> for leaf in abjad.iterate(staff).leaves():
        ...     leaf
        ...
        Note("c'4")
        Note("e'4")
        Note("d'4")
        Note("f'4")

    """
    import abjad
    if client is not None:
        return abjad.Iteration(client=client)
    expression = abjad.Expression()
    expression = expression.iterate()
    return expression



# import order: tools, materials, segments.
# makes it possible for materials to import tools.
# makes it possible for segments to import both.
from {score_package_name}.tools import *
from {score_package_name}.materials import *
from {score_package_name} import segments



from abjad.system.AbjadObject import AbjadObject


# TODO: should not inherit from AbjadObject because no slots
class LilyPondEvent(AbjadObject):
    """
    Model of an arbitrary event in LilyPond.

    Not composer-safe.

    Used internally by LilyPondParser.
    """

    ### INITIALIZER ###

    def __init__(self, name=None, **keywords):
        self.name = name
        for k, v in keywords.items():
            if k != 'name':
                setattr(self, k, v)

    ### SPECIAL METHODS ###

    def __str__(self):
        """
        Gets string representation of LilyPond event.

        Returns string.
        """
        result = repr(self.name)
        for key in self.__dict__:
            if key == 'name':
                continue
            result += ', {} = {!r}'.format(key, getattr(self, key))
        return result

    def __repr__(self):
        """
        Gets interpreter representation of LilyPond event.

        Returns string.
        """
        return '{}({})'.format(type(self).__name__, str(self))



import typing
from abjad.system.AbjadValueObject import AbjadValueObject
from abjad.lilypondnames.LilyPondTweakManager import LilyPondTweakManager
from abjad.system.FormatSpecification import FormatSpecification
from abjad.system.LilyPondFormatBundle import LilyPondFormatBundle
from abjad.system.StorageFormatManager import StorageFormatManager


class LilyPondLiteral(AbjadValueObject):
    r"""
    LilyPond literal.

    ..  container:: example

        Dotted slur:

        >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
        >>> slur = abjad.Slur()
        >>> abjad.attach(slur, staff[:])
        >>> literal = abjad.LilyPondLiteral(r'\slurDotted')
        >>> abjad.attach(literal, staff[0])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                \slurDotted
                c'8
                (
                d'8
                e'8
                f'8
                )
            }

    ..  container:: example

        Use the absolute before and absolute after format slots like this:

        >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
        >>> abjad.attach(abjad.Slur(), staff[:])
        >>> literal = abjad.LilyPondLiteral(r'\slurDotted')
        >>> abjad.attach(literal, staff[0])
        >>> literal = abjad.LilyPondLiteral('', format_slot='absolute_before')
        >>> abjad.attach(literal, staff[0])
        >>> literal = abjad.LilyPondLiteral(
        ...     '% before all formatting',
        ...     format_slot='absolute_before',
        ...     )
        >>> abjad.attach(literal, staff[0])
        >>> literal = abjad.LilyPondLiteral('', format_slot='absolute_after')
        >>> abjad.attach(literal, staff[-1])
        >>> abjad.show(staff) # doctest: +SKIP

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
            <BLANKLINE>
                % before all formatting
                \slurDotted
                c'8
                (
                d'8
                e'8
                f'8
                )
            <BLANKLINE>
            }

    ..  container:: example

        LilyPond literals can be tagged:

        >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
        >>> abjad.attach(abjad.Slur(), staff[:])
        >>> literal = abjad.LilyPondLiteral(r'\slurDotted')
        >>> abjad.attach(literal, staff[0], tag='RED')
        >>> abjad.show(staff) # doctest: +SKIP

        >>> abjad.f(staff)
        \new Staff
        {
            \slurDotted %! RED
            c'8
            (
            d'8
            e'8
            f'8
            )
        }

    ..  container:: example

        Multiline input is allowed:

        >>> staff = abjad.Staff("c'8 d'8 e'8 f'8")
        >>> abjad.attach(abjad.Slur(), staff[:])
        >>> lines = [
        ...     r'\stopStaff',
        ...     r'\startStaff',
        ...     r'\once \override Staff.StaffSymbol.color = #red',
        ...     ]
        >>> literal = abjad.LilyPondLiteral(lines)
        >>> abjad.attach(literal, staff[2], tag='RED')
        >>> abjad.show(staff) # doctest: +SKIP

        >>> abjad.f(staff)
        \new Staff
        {
            c'8
            (
            d'8
            \stopStaff %! RED
            \startStaff %! RED
            \once \override Staff.StaffSymbol.color = #red %! RED
            e'8
            f'8
            )
        }

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_argument',
        '_directed',
        '_format_slot',
        '_tweaks',
        )

    _allowable_format_slots = (
        'absolute_after',
        'absolute_before',
        'after',
        'before',
        'closing',
        'opening',
        )

    _can_attach_to_containers = True

    _format_leaf_children = False

    ### INITIALIZER ###

    def __init__(
        self,
        argument: typing.Union[str, typing.List[str]] = '',
        format_slot: str = 'opening',
        *,
        directed: bool = None,
        tweaks: typing.Union[
            typing.List[typing.Tuple], LilyPondTweakManager] = None,
        ) -> None:
        self._argument = argument
        assert format_slot in self._allowable_format_slots, repr(format_slot)
        self._format_slot = format_slot
        if directed is not None:
            directed = bool(directed)
        self._directed = directed
        self._tweaks = None
        LilyPondTweakManager.set_tweaks(self, tweaks)

    ### SPECIAL METHODS ###

    def __format__(self, format_specification='') -> str:
        """
        Formats LilyPond literal.
        """
        if format_specification in ('', 'storage'):
            return StorageFormatManager(self).get_storage_format()
        assert format_specification == 'lilypond'
        return str(self.argument)

    ### PRIVATE METHODS ###

    def _get_format_pieces(self):
        if isinstance(self.argument, str):
            return [self.argument]
        assert isinstance(self.argument, list)
        return self.argument[:]

    def _get_format_specification(self):
        names = []
        if not self.format_slot == 'opening':
            names.append('format_slot')
        return FormatSpecification(
            client=self,
            storage_format_args_values=[self.argument],
            storage_format_kwargs_names=names,
            storage_format_is_indented=False,
            )

    def _get_lilypond_format_bundle(self, component=None):
        bundle = LilyPondFormatBundle()
        format_slot = bundle.get(self.format_slot)
        if self.tweaks:
            tweaks = self.tweaks._list_format_contributions(
                directed=self.directed,
                )
            format_slot.commands.extend(tweaks)
        pieces = self._get_format_pieces()
        format_slot.commands.extend(pieces)
        return bundle

    ### PUBLIC PROPERTIES ###

    @property
    def argument(self) -> typing.Union[str, typing.List[str]]:
        r"""
        Gets argument of LilyPond literal.

        ..  container:: example

            >>> literal = abjad.LilyPondLiteral(r'\slurDotted')
            >>> literal.argument
            '\\slurDotted'

        """
        return self._argument

    @property
    def directed(self) -> typing.Optional[bool]:
        r"""
        Is true when literal is directed.

        ..  container:: example

            Directed literal:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> literal = abjad.LilyPondLiteral(r'\f', 'after', directed=True)
            >>> abjad.tweak(literal).color = 'blue'
            >>> abjad.tweak(literal).dynamic_line_spanner.staff_padding = 5
            >>> abjad.attach(literal, staff[0])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'4
                    - \tweak DynamicLineSpanner.staff-padding #5
                    - \tweak color #blue
                    \f
                    d'4
                    e'4
                    f'4
                }

        ..  container:: example

            Nondirected literal:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> literal = abjad.LilyPondLiteral(
            ...     r'\breathe',
            ...     'after',
            ...     directed=False,
            ...     )
            >>> abjad.tweak(literal).color = 'blue'
            >>> abjad.attach(literal, staff[0])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'4
                    \tweak color #blue
                    \breathe
                    d'4
                    e'4
                    f'4
                }

        Proper use of the ``directed`` property entails searching the LilyPond
        docs to understand whether LilyPond treats any particular command as
        directed or not. Most LilyPond commands are directed. LilyPond insists
        that a few commands (include ``\breathe``, ``\key``, ``\mark``) must
        not be directed.
        """
        return self._directed

    @property
    def format_slot(self) -> str:
        """
        Gets format slot of LilyPond literal.

        ..  container:: example

            >>> literal = abjad.LilyPondLiteral(r'\slurDotted')
            >>> literal.format_slot
            'opening'

        """
        return self._format_slot

    @property
    def tweaks(self) -> typing.Optional[LilyPondTweakManager]:
        r"""
        Gets tweaks.

        ..  container:: example

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> literal = abjad.LilyPondLiteral(
            ...     r'\f',
            ...     'after',
            ...     directed=True,
            ...     tweaks=[('color', 'blue')],
            ...     )
            >>> abjad.attach(literal, staff[0])
            >>> abjad.show(staff) # doctest: +SKIP

            ..  docs::

                >>> abjad.f(staff)
                \new Staff
                {
                    c'4
                    - \tweak color #blue
                    \f
                    d'4
                    e'4
                    f'4
                }

        """
        return self._tweaks

    ### PUBLIC METHODS ###

    @staticmethod
    def list_allowable_format_slots() -> typing.Tuple[str, ...]:
        """
        Lists allowable format slots.

        ..  container:: example

            >>> for slot in abjad.LilyPondLiteral.list_allowable_format_slots():
            ...     slot
            ...
            'absolute_after'
            'absolute_before'
            'after'
            'before'
            'closing'
            'opening'

        """
        return LilyPondLiteral._allowable_format_slots



"""
Tools for high-level score modeling.
"""

from .ScoreTemplate import ScoreTemplate
from .GroupedRhythmicStavesScoreTemplate import \
    GroupedRhythmicStavesScoreTemplate
from .GroupedStavesScoreTemplate import GroupedStavesScoreTemplate
from .Job import Job
from .Line import Line
from .Momento import Momento
from .Section import Section
from .Part import Part
from .PartAssignment import PartAssignment
from .PartManifest import PartManifest
from .Path import Path
from .PersistentOverride import PersistentOverride
from .SegmentMaker import SegmentMaker
from .StringOrchestraScoreTemplate import StringOrchestraScoreTemplate
from .StringQuartetScoreTemplate import StringQuartetScoreTemplate
from .TwoStaffPianoScoreTemplate import TwoStaffPianoScoreTemplate



import typing
from abjad.timespans import TimespanList
from abjad import mathtools
from abjad.system.AbjadObject import AbjadObject
from abjad.utilities.OrderedDict import OrderedDict
from abjad.utilities.String import String
from abjad.lilypondfile.LilyPondFile import LilyPondFile
from abjad.core.Container import Container
from abjad.core.Context import Context
from abjad.core.Score import Score
from abjad.core.Staff import Staff
from abjad.core.Voice import Voice
from abjad.system.StorageFormatManager import StorageFormatManager
from abjad.system.TestManager import TestManager
from abjad.top.inspect import inspect
from abjad.top.iterate import iterate
from .PartAssignment import PartAssignment
from .Path import Path


class SegmentMaker(AbjadObject):
    """
    Segment-maker.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Segment-makers'

    __slots__ = (
        '_container_to_part_assignment',
        '_environment',
        '_lilypond_file',
        '_metadata',
        '_previous_metadata',
        '_score',
        '_segment_directory',
        )

    ### INITIALIZER ###

    def __init__(self):
        self._container_to_part_assignment: typing.Optional[OrderedDict] = None
        self._environment: typing.Optional[str] = None
        self._lilypond_file: typing.Optional[LilyPondFile] = None
        self._metadata: typing.Optional[OrderedDict] = None
        self._previous_metadata: typing.Optional[OrderedDict] = None
        self._score: typing.Optional[Score] = None
        self._segment_directory: typing.Optional[Path] = None

    ### SPECIAL METHODS ###

    def __eq__(self, expr):
        """
        Is true if ``expr`` is a segment-maker with equivalent properties.
        """
        return TestManager.compare_objects(self, expr)

    def __hash__(self):
        """
        Hashes segment-maker.
        """
        hash_values = StorageFormatManager(self).get_hash_values()
        return hash(hash_values)

    def __illustrate__(self, **keywords) -> LilyPondFile:
        """
        Illustrates segment-maker.
        """
        lilypond_file = self.run(**keywords)
        return lilypond_file

    ### PRIVATE METHODS ###

    def _add_container_identifiers(self):
        if (self.environment == 'docs' and
            not getattr(self, 'test_container_identifiers', False)):
            return
        segment_name = self.segment_name or ''
        segment_name = String(segment_name).to_segment_lilypond_identifier()
        contexts = []
        try:
            context = self.score['GlobalSkips']
            contexts.append(context)
        except ValueError:
            pass
        try:
            context = self.score['GlobalRests']
            contexts.append(context)
        except ValueError:
            pass
        for voice in iterate(self.score).components(Voice):
            contexts.append(voice)
        container_to_part_assignment = OrderedDict()
        for context in contexts:
            if segment_name:
                context_identifier = f'{segment_name}_{context.name}'
            else:
                context_identifier = context.name
            context.identifier = f'%*% {context_identifier}'
            part_container_count = 0
            for container in iterate(context).components(Container):
                if not container.identifier:
                    continue
                if container.identifier.startswith('%*% Part'):
                    part_container_count += 1
                    globals_ = globals()
                    part = container.identifier.strip('%*% ')
                    part = eval(part, globals_)
                    suffix = String().base_26(part_container_count).lower()
                    container_identifier = f'{context_identifier}_{suffix}'
                    container_identifier = String(container_identifier)
                    assert container_identifier.is_lilypond_identifier()
                    assert container_identifier not in \
                        container_to_part_assignment
                    timespan = inspect(container).timespan()
                    pair = (part, timespan)
                    container_to_part_assignment[container_identifier] = pair
                    container.identifier = f'%*% {container_identifier}'
        for staff in iterate(self.score).components(Staff):
            if segment_name:
                context_identifier = f'{segment_name}_{staff.name}'
            else:
                context_identifier = staff.name
            staff.identifier = f'%*% {context_identifier}'
        self._container_to_part_assignment = container_to_part_assignment

    def _check_duplicate_part_assignments(self):
        dictionary = self._container_to_part_assignment
        if not dictionary:
            return
        if not self.score_template:
            return
        part_manifest = self.score_template.part_manifest
        if not part_manifest:
            return
        part_to_timespans = OrderedDict()
        for identifier, (part_assignment, timespan) in dictionary.items():
            for part in part_manifest.expand(part_assignment):
                if part.name not in part_to_timespans:
                    part_to_timespans[part.name] = []
                part_to_timespans[part.name].append(timespan)
        messages = []
        for part_name, timespans in part_to_timespans.items():
            if len(timespans) <= 1:
                continue
            timespan_list = TimespanList(timespans)
            if timespan_list.compute_logical_and():
                message = f'  Part {part_name!r} is assigned'
                message += ' to overlapping containers ...'
                messages.append(message)
        if messages:
            message = '\n' + '\n'.join(messages)
            raise Exception(message)

    def _make_global_context(self):
        global_rests = Context(
            lilypond_type='GlobalRests',
            name='GlobalRests',
            )
        global_skips = Context(
            lilypond_type='GlobalSkips',
            name='GlobalSkips',
            )
        global_context = Context(
            [global_rests, global_skips],
            lilypond_type='GlobalContext',
            is_simultaneous=True,
            name='GlobalContext',
            )
        return global_context

    def _make_lilypond_file(self, midi=False):
        pass

    ### PUBLIC PROPERTIES ###

    @property
    def environment(self) -> typing.Optional[str]:
        """
        Gets environment.
        """
        return self._environment

    @property
    def metadata(self) -> typing.Optional[OrderedDict]:
        """
        Gets segment metadata after run.
        """
        return self._metadata

    @property
    def score(self) -> typing.Optional[Score]:
        """
        Gets score.
        """
        return self._score

    @property
    def segment_directory(self) -> typing.Optional[Path]:
        """
        Gets segment directory.
        """
        return self._segment_directory

    @property
    def segment_name(self) -> typing.Optional[str]:
        """
        Gets segment name.
        """
        if self.segment_directory is not None:
            return self.segment_directory.name
        return None

    ### PUBLIC METHODS ###

    def run(
        self,
        deactivate: typing.List[str] = None,
        environment: str = None,
        metadata: OrderedDict = None,
        midi: bool = None,
        previous_metadata: OrderedDict = None,
        remove: typing.List[str] = None,
        segment_directory: Path = None,
        ) -> LilyPondFile:
        """
        Runs segment-maker.
        """
        self._metadata = OrderedDict(metadata)
        self._previous_metadata = OrderedDict(previous_metadata)
        lilypond_file = self._make_lilypond_file(midi=midi)
        self._lilypond_file = lilypond_file
        assert isinstance(self._lilypond_file, LilyPondFile)
        return self._lilypond_file



from abjad.system.AbjadObject import AbjadObject
from abjad.system.LilyPondFormatManager import \
    LilyPondFormatManager
from abjad.system.StorageFormatManager import StorageFormatManager


class Block(AbjadObject):
    """
    A LilyPond file block.

    ..  container:: example

        REGRESSION. Blocks remember attribute assignment order.

        Here right margin precedes left margin even though left margin
        alphabetizes before right margin:

        >>> block = abjad.Block(name='paper')
        >>> block.right_margin = abjad.LilyPondDimension(2, 'cm')
        >>> block.left_margin = abjad.LilyPondDimension(2, 'cm')
        >>> block
        <Block(name='paper')>

        >>> abjad.f(block)
        \paper {
            right-margin = 2\cm
            left-margin = 2\cm
        }

    ..  container:: example

        >>> block = abjad.Block(name='score')
        >>> markup = abjad.Markup('foo')
        >>> block.items.append(markup)
        >>> block
        <Block(name='score')>

        >>> abjad.f(block)
        \score {
            {
                \markup { foo }
            }
        }

    """

    ### INITIALIZER ###

    def __init__(self, name='score'):
        assert isinstance(name, str), repr(name)
        self._name = name
        escaped_name = rf'\{name}'
        self._escaped_name = escaped_name
        self._items = []
        self._public_attribute_names = []

    ### SPECIAL METHODS ###

    def __delattr__(self, name) -> None:
        """
        Deletes block attribute with ``name``.

        ..  container:: example

            >>> header_block = abjad.Block(name='header')
            >>> header_block.tagline = False
            >>> header_block.tagline
            False

            >>> delattr(header_block, 'tagline')
            >>> hasattr(header_block, 'tagline')
            False

        """
        self._public_attribute_names.remove(name)
        object.__delattr__(self, name)

    def __format__(self, format_specification=''):
        """
        Formats block.

        Returns string.
        """
        if format_specification in ('', 'lilypond'):
            return self._get_lilypond_format()
        else:
            assert format_specification == 'storage'
            return StorageFormatManager(self).get_storage_format()

    def __getitem__(self, name):
        """
        Gets item with ``name``.

        ..  container:: example

            Gets score with name ``'Red Example Score'`` in score block:

            >>> block = abjad.Block(name='score')
            >>> score = abjad.Score(name='Red Example Score')
            >>> block.items.append(score)

            >>> block['Red Example Score']
            Score(is_simultaneous=True, name='Red Example Score')

        Returns item.

        Raises key error when no item with ``name`` is found.
        """
        for item in self.items:
            if getattr(item, 'name', None) == name:
                return item
        raise KeyError

    def __setattr__(self, name, value):
        """
        Sets block ``name`` to ``value``.

        Returns none.
        """
        if (
            not name.startswith('_') and
            name not in self._public_attribute_names
            ):
            self._public_attribute_names.append(name)
        object.__setattr__(self, name, value)

    def __setstate__(self, state):
        """
        Sets state of block.

        Returns none.
        """
        if not hasattr(self, '_public_attribute_names'):
            self._public_attribute_names = []
        for key, value in state.items():
            setattr(self, key, value)

    ### PRIVATE PROPERTIES ###

    @property
    def _formatted_context_blocks(self):
        import abjad
        result = []
        context_blocks = []
        for item in self.items:
            if isinstance(item, abjad.ContextBlock):
                context_blocks.append(item)
        for context_block in context_blocks:
            result.extend(context_block._get_format_pieces())
        return result

    ### PRIVATE METHODS ###

    def _format_item(self, item, depth=1):
        indent = LilyPondFormatManager.indent * depth
        result = []
        if isinstance(item, (list, tuple)):
            result.append(indent + '{')
            depth_ = depth + 1
            for x in item:
                pieces = self._format_item(x, depth=depth_)
                result.extend(pieces)
            result.append(indent + '}')
        elif isinstance(item, str):
            string = indent + item
            result.append(string)
        elif '_get_format_pieces' in dir(item):
            try:
                pieces = item._get_format_pieces()
            except TypeError:
                pieces = item._get_format_pieces()
            pieces = (indent + item for item in pieces)
            result.extend(pieces)
        return result

    def _get_format_pieces(self):
        import abjad
        indent = LilyPondFormatManager.indent
        result = []
        if (not self._get_formatted_user_attributes() and
            not getattr(self, 'contexts', None) and
            not getattr(self, 'context_blocks', None) and
            not len(self.items)):
            if self.name == 'score':
                return ''
            string = '{} {{}}'.format(self._escaped_name)
            result.append(string)
            return result
        string = '{} {{'.format(self._escaped_name)
        result.append(string)
        for item in self.items:
            if isinstance(item, abjad.ContextBlock):
                continue
            if isinstance(item, (abjad.Leaf, abjad.Markup)):
                item = [item]
            result.extend(self._format_item(item))
        formatted_attributes = self._get_formatted_user_attributes()
        formatted_attributes = [indent + _ for _ in formatted_attributes]
        result.extend(formatted_attributes)
        formatted_context_blocks = getattr(
            self, '_formatted_context_blocks', [])
        formatted_context_blocks = [
            indent + _ for _ in formatted_context_blocks]
        result.extend(formatted_context_blocks)
        result.append('}')
        return result

    def _get_format_specification(self):
        import abjad
        return abjad.FormatSpecification(
            client=self,
            repr_is_bracketed=True,
            repr_is_indented=False,
            )

    def _get_formatted_user_attributes(self):
        import abjad
        result = []
        #prototype = (abjad.Scheme, abjad.LilyPondLiteral)
        prototype = abjad.Scheme
        for value in self.items:
            if isinstance(value, prototype):
                result.append(format(value, 'lilypond'))
        prototype = (
            abjad.Scheme,
            abjad.LilyPondDimension,
            #abjad.LilyPondLiteral,
            )
        for key in self._public_attribute_names:
            assert not key.startswith('_'), repr(key)
            value = getattr(self, key)
            # format subkeys via double underscore
            formatted_key = key.split('__')
            for i, k in enumerate(formatted_key):
                formatted_key[i] = k.replace('_', '-')
                if 0 < i:
                    string = "#'{}".format(formatted_key[i])
                    formatted_key[i] = string
            formatted_key = ' '.join(formatted_key)
            # format value
            if isinstance(value, abjad.Markup):
                formatted_value = value._get_format_pieces()
            elif isinstance(value, prototype):
                formatted_value = [format(value, 'lilypond')]
            else:
                formatted_value = abjad.Scheme(value)
                formatted_value = format(formatted_value, 'lilypond')
                formatted_value = [formatted_value]
            setting = '{!s} = {!s}'
            setting = setting.format(
                formatted_key,
                formatted_value[0],
                )
            result.append(setting)
            result.extend(formatted_value[1:])
        return result

    def _get_lilypond_format(self):
        return '\n'.join(self._get_format_pieces())

    ### PUBLIC PROPERTIES ###

    @property
    def items(self):
        r"""
        Gets items in block.

        ..  container:: example

            >>> block = abjad.Block(name='score')
            >>> markup = abjad.Markup('foo')
            >>> block.items.append(markup)

            >>> block.items
            [Markup(contents=['foo'])]

        ..  container:: example

            Accepts strings:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> score_block = abjad.Block(name='score')
            >>> score_block.items.append('<<')
            >>> score_block.items.append(r'{ \include "layout.ly" }')
            >>> score_block.items.append(staff)
            >>> score_block.items.append('>>')
            >>> lilypond_file = abjad.LilyPondFile(
            ...     lilypond_language_token=False,
            ...     lilypond_version_token=False,
            ...     )
            >>> lilypond_file.items.append(score_block)

            >>> abjad.f(lilypond_file)
            \score {
                <<
                { \include "layout.ly" }
                \new Staff
                {
                    c'4
                    d'4
                    e'4
                    f'4
                }
                >>
            }

        Returns list.
        """
        return self._items

    @property
    def name(self):
        """
        Gets name of block.

        ..  container:: example

            >>> block = abjad.Block(name='score')
            >>> markup = abjad.Markup('foo')
            >>> block.items.append(markup)

            >>> block.name
            'score'

        Returns string.
        """
        return self._name

    ### PUBLIC METHODS ###

    def empty(self):
        """
        Is true when block contains no items and has no user attributes.

        Returns true or false.
        """
        if not self.items and not self._get_formatted_user_attributes():
            return True
        return False



from abjad.system.AbjadObject import AbjadObject


class LilyPondFormatBundle(AbjadObject):
    """
    LilyPond format bundle.

    Transient class created to hold the collection of all
    format contributions generated on behalf of a single component.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'LilyPond formatting'

    __slots__ = (
        '_absolute_after',
        '_absolute_before',
        '_after',
        '_before',
        '_closing',
        '_context_settings',
        '_grob_overrides',
        '_grob_reverts',
        '_opening',
        )

    ### INITIALIZER ###

    def __init__(self):
        import abjad
        self._absolute_after = abjad.SlotContributions()
        self._absolute_before = abjad.SlotContributions()
        self._before = abjad.SlotContributions()
        self._after = abjad.SlotContributions()
        self._opening = abjad.SlotContributions()
        self._closing = abjad.SlotContributions()
        self._context_settings = []
        self._grob_overrides = []
        self._grob_reverts = []

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        import abjad
        slot_contribution_names = (
            'absolute_before',
            'absolute_after',
            'before',
            'after',
            'opening',
            'closing',
            )
        grob_contribution_names = (
            'context_settings',
            'grob_overrides',
            'grob_reverts',
            )
        names = [_ for _ in slot_contribution_names
            if getattr(self, _).has_contributions]
        names.extend(_ for _ in grob_contribution_names
            if getattr(self, _))
        return abjad.FormatSpecification(
            client=self,
            storage_format_kwargs_names=names,
            )

    ### PUBLIC METHODS ###

    def get(self, identifier):
        """
        Gets ``identifier``.

        Returns format contributions object or list.
        """
        return getattr(self, identifier)

    def sort_overrides(self):
        """
        Makes each slot immutable.

        Returns none.
        """
        self._context_settings = tuple(sorted(set(self.context_settings)))
        self._grob_overrides = tuple(sorted(set(self.grob_overrides)))
        self._grob_reverts = tuple(sorted(set(self.grob_reverts)))

    def tag_format_contributions(self, tag, deactivate=None):
        """
        Tags format contributions with string ``tag``.

        Returns none.
        """
        import abjad
        self.absolute_before.tag(tag, deactivate)
        self.absolute_after.tag(tag, deactivate)
        self.before.tag(tag, deactivate)
        self.after.tag(tag, deactivate)
        self.opening.tag(tag, deactivate)
        self.closing.tag(tag, deactivate)
        self._context_settings = abjad.LilyPondFormatManager.tag(
            self.context_settings,
            tag,
            deactivate,
            )
        self._grob_overrides = abjad.LilyPondFormatManager.tag(
            self.grob_overrides,
            tag,
            deactivate,
            )
        self._grob_reverts = abjad.LilyPondFormatManager.tag(
            self.grob_reverts,
            tag,
            deactivate,
            )

    def update(self, format_bundle):
        """
        Updates format bundle with all format contributions in
        ``format_bundle``.

        Returns none.
        """
        if hasattr(format_bundle, '_get_lilypond_format_bundle'):
            format_bundle = format_bundle._get_lilypond_format_bundle()
        assert isinstance(format_bundle, type(self))
        self.absolute_before.update(format_bundle.absolute_before)
        self.absolute_after.update(format_bundle.absolute_after)
        self.before.update(format_bundle.before)
        self.after.update(format_bundle.after)
        self.opening.update(format_bundle.opening)
        self.closing.update(format_bundle.closing)
        self.context_settings.extend(format_bundle.context_settings)
        self.grob_overrides.extend(format_bundle.grob_overrides)
        self.grob_reverts.extend(format_bundle.grob_reverts)

    ### PUBLIC PROPERTIES ###

    @property
    def absolute_after(self):
        """
        Aboslute after slot contributions.

        Returns slot contributions object.
        """
        return self._absolute_after

    @property
    def absolute_before(self):
        """
        Absolute before slot contributions.

        Returns slot contributions object.
        """
        return self._absolute_before

    @property
    def after(self):
        """
        After slot contributions.

        Returns slot contributions object.
        """
        return self._after

    @property
    def before(self):
        """
        Before slot contributions.

        Returns slot contributions object.
        """
        return self._before

    @property
    def closing(self):
        """
        Closing slot contributions.

        Returns slot contributions object.
        """
        return self._closing

    @property
    def context_settings(self):
        """
        Context setting format contributions.

        Returns list.
        """
        return self._context_settings

    @property
    def grob_overrides(self):
        """
        Grob override format contributions.

        Returns list.
        """
        return self._grob_overrides

    @property
    def grob_reverts(self):
        """
        Grob revert format contributions.

        Returns list.
        """
        return self._grob_reverts

    @property
    def opening(self):
        """
        Opening slot contributions.

        Returns slot contributions object.
        """
        return self._opening

#    @property
#    def right(self):
#        """
#        Right slot contributions.
#
#        Returns slot contributions object.
#        """
#        return self._right



import typing
from abjad.system.AbjadValueObject import AbjadValueObject
from abjad.scheme import Scheme
from abjad.system.LilyPondFormatBundle import LilyPondFormatBundle


class LilyPondContextSetting(AbjadValueObject):
    r"""
    LilyPond context setting.

    ..  container:: example

        >>> context_setting = abjad.LilyPondContextSetting(
        ...    lilypond_type='Score',
        ...    context_property='autoBeaming',
        ...    value=False,
        ...    )

        >>> print('\n'.join(context_setting.format_pieces))
        \set Score.autoBeaming = ##f

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_context_property',
        '_lilypond_type',
        '_is_unset',
        '_value',
        )

    _format_leaf_children = False

    ### INITIALIZER ###

    def __init__(
        self,
        lilypond_type: str = None,
        context_property: str = 'autoBeaming',
        is_unset: bool = False,
        value: typing.Any = False,
        ) -> None:
        if lilypond_type is not None:
            lilypond_type = str(lilypond_type)
        self._lilypond_type = lilypond_type
        assert isinstance(context_property, str) and context_property
        self._context_property = context_property
        if is_unset is not None:
            is_unset = bool(is_unset)
        self._is_unset = is_unset
        self._value = value

    ### SPECIAL METHODS ###

    def __eq__(self, argument) -> bool:
        """
        Is true when ``argument`` is a LilyPond context setting with
        equivalent keyword values.
        """
        return super().__eq__(argument)

    def __hash__(self) -> int:
        """
        Hashes LilyPond context setting.

        Redefined in tandem with __eq__.
        """
        return super().__hash__()

    ### PRIVATE METHODS ###

    def _get_lilypond_format_bundle(self, component=None):
        bundle = LilyPondFormatBundle()
        string = '\n'.join(self.format_pieces)
        bundle.context_settings.append(string)
        return bundle

    ### PUBLIC PROPERTIES ###

    @property
    def lilypond_type(self) -> typing.Optional[str]:
        """
        Gets LilyPond type.
        """
        return self._lilypond_type

    @property
    def context_property(self) -> str:
        """
        Gets LilyPond context property name.
        """
        return self._context_property

    @property
    def format_pieces(self) -> typing.Tuple[str, ...]:
        r"""
        Gets LilyPond context setting ``\set`` or ``\unset`` format pieces.
        """
        result = []
        if not self.is_unset:
            result.append(r'\set')
        else:
            result.append(r'\unset')
        if self.lilypond_type is not None:
            string = f'{self.lilypond_type}.{self.context_property}'
            result.append(string)
        else:
            result.append(self.context_property)
        result.append('=')
        string = Scheme.format_embedded_scheme_value(self.value)
        value_pieces = string.split('\n')
        result.append(value_pieces[0])
        result[:] = [' '.join(result)]
        result.extend(value_pieces[1:])
        return tuple(result)

    @property
    def is_unset(self) -> typing.Optional[bool]:
        """
        Is true if context setting unsets its value.
        """
        return self._is_unset

    @property
    def value(self) -> typing.Any:
        """
        Gets value of LilyPond context setting.
        """
        return self._value



from abjad.system.AbjadValueObject import AbjadValueObject


class FormatSpecification(AbjadValueObject):
    """
    Format specification.
    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Storage formatting'

    __slots__ = (
        '_client',
        '_coerce_for_equality',
        '_repr_args_values',
        '_repr_is_bracketed',
        '_repr_is_indented',
        '_repr_kwargs_names',
        '_repr_text',
        '_storage_format_args_values',
        '_storage_format_forced_override',
        '_storage_format_is_bracketed',
        '_storage_format_is_indented',
        '_storage_format_kwargs_names',
        '_storage_format_text',
        '_template_names',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        client=None,
        coerce_for_equality=None,
        repr_args_values=None,
        repr_is_bracketed=None,
        repr_is_indented=None,
        repr_kwargs_names=None,
        repr_text=None,
        storage_format_args_values=None,
        storage_format_forced_override=None,
        storage_format_is_bracketed=None,
        storage_format_is_indented=True,
        storage_format_kwargs_names=None,
        storage_format_text=None,
        template_names=None,
        ):
        self._client = client
        self._coerce_for_equality = self._coerce_boolean(coerce_for_equality)
        self._repr_args_values = self._coerce_tuple(repr_args_values)
        self._repr_is_bracketed = self._coerce_boolean(repr_is_bracketed)
        self._repr_is_indented = self._coerce_boolean(repr_is_indented)
        self._repr_kwargs_names = self._coerce_tuple(repr_kwargs_names)
        self._repr_text = self._coerce_string(repr_text)
        self._storage_format_args_values = self._coerce_tuple(
            storage_format_args_values)
        self._storage_format_forced_override = storage_format_forced_override
        self._storage_format_is_bracketed = self._coerce_boolean(
            storage_format_is_bracketed)
        self._storage_format_is_indented = self._coerce_boolean(
            storage_format_is_indented)
        self._storage_format_kwargs_names = self._coerce_tuple(
            storage_format_kwargs_names)
        self._storage_format_text = self._coerce_string(storage_format_text)
        self._template_names = self._coerce_tuple(template_names)

    ### PRIVATE METHODS ###

    def _coerce_boolean(self, value):
        if value is not None:
            return bool(value)

    def _coerce_string(self, value):
        if value is not None:
            return str(value)

    def _coerce_tuple(self, value):
        if value is not None:
            return tuple(value)

    ### PUBLIC PROPERTIES ###

    @property
    def client(self):
        return self._client

    @property
    def coerce_for_equality(self):
        return self._coerce_for_equality

    @property
    def repr_args_values(self):
        return self._repr_args_values

    @property
    def repr_is_bracketed(self):
        return self._repr_is_bracketed

    @property
    def repr_is_indented(self):
        return self._repr_is_indented

    @property
    def repr_kwargs_names(self):
        return self._repr_kwargs_names

    @property
    def repr_text(self):
        return self._repr_text

    @property
    def storage_format_args_values(self):
        return self._storage_format_args_values

    @property
    def storage_format_forced_override(self):
        return self._storage_format_forced_override

    @property
    def storage_format_is_bracketed(self):
        return self._storage_format_is_bracketed

    @property
    def storage_format_is_indented(self):
        return self._storage_format_is_indented

    @property
    def storage_format_kwargs_names(self):
        return self._storage_format_kwargs_names

    @property
    def storage_format_text(self):
        return self._storage_format_text

    @property
    def template_names(self):
        return self._template_names



# TODO: move to Sequence
def cumulative_sums_pairwise(argument):
    """
    Gets pairwise cumulative sums of ``argument`` from zero.

    ..  container:: example

        >>> abjad.mathtools.cumulative_sums_pairwise([1, 2, 3, 4, 5, 6])
        [(0, 1), (1, 3), (3, 6), (6, 10), (10, 15), (15, 21)]

    Returns pairs in new object of ``argument`` type.
    """
    import abjad
    sums = abjad.mathtools.cumulative_sums(argument)
    pairs = abjad.sequence(sums).nwise()
    return type(argument)([tuple(_) for _ in pairs])



import typing
from abjad.system.LilyPondFormatManager import LilyPondFormatManager
from abjad.utilities.String import String
from .LilyPondNameManager import LilyPondNameManager


class LilyPondGrobNameManager(LilyPondNameManager):
    """
    LilyPond grob name manager.

    ..  container:: example

        LilyPondGrobNameManager instances are created by the
        ``abjad.override()`` factory function:

        >>> note = abjad.Note("c'4")
        >>> abjad.override(note)
        LilyPondGrobNameManager()

    """

    ### SPECIAL METHODS ###

    def __getattr__(self, name) -> typing.Union[
        LilyPondNameManager, 'LilyPondGrobNameManager'
        ]:
        r"""
        Gets LilyPondNameManager (or LilyPondGrobNameManager) keyed to 
        ``name``.

        ..  container:: example

            Somewhat confusingly, getting a grob name returns a
            LilyPondNameManager:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.override(staff[0]).note_head
            LilyPondNameManager()

            While getting a context name returns a LilyPondGrobNameManager:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.override(staff[0]).staff
            LilyPondGrobNameManager()

            Which can then be deferenced to get a LilyPondNameManager:

            >>> staff = abjad.Staff("c'4 d' e' f'")
            >>> abjad.override(staff[0]).staff.note_head
            LilyPondNameManager()

        Note that the dot-chained user syntax is unproblematic. But the class
        of each manager returned in the chain is likely to be surprising at
        first encounter.
        """
        from abjad.ly import contexts
        from abjad.ly import grob_interfaces
        camel_name = String(name).to_upper_camel_case()
        if name.startswith('_'):
            try:
                return vars(self)[name]
            except KeyError:
                type_name = type(self).__name__
                message = '{type_name!r} object has no attribute: {name!r}.'
                raise AttributeError(message)
        elif camel_name in contexts:
            try:
                return vars(self)['_' + name]
            except KeyError:
                context = LilyPondGrobNameManager()
                vars(self)['_' + name] = context
                return context
        elif camel_name in grob_interfaces:
            try:
                return vars(self)[name]
            except KeyError:
                vars(self)[name] = LilyPondNameManager()
                return vars(self)[name]
        else:
            try:
                return vars(self)[name]
            except KeyError:
                type_name = type(self).__name__
                message = f'{type_name!r} object has no attribute: {name!r}.'
                raise AttributeError(message)

    def __setattr__(self, attribute, value) -> None:
        """
        Sets attribute ``attribute`` of grob name manager to ``value``.
        """
        # make sure attribute name is valid grob name before setting value
        object.__setattr__(self, attribute, value)

    ### PRIVATE METHODS ###

    def _get_attribute_tuples(self):
        result = []
        for name, value in vars(self).items():
            if type(value) is LilyPondNameManager:
                grob, grob_proxy = name, value
                pairs = iter(vars(grob_proxy).items())
                for attribute, value in pairs:
                    triple = (grob, attribute, value)
                    result.append(triple)
            else:
                context, context_proxy = name.strip('_'), value
                for grob, grob_proxy in vars(context_proxy).items():
                    pairs = iter(vars(grob_proxy).items())
                    for attribute, value in pairs:
                        quadruple = (
                            context,
                            grob,
                            attribute,
                            value,
                            )
                        result.append(quadruple)
        return tuple(result)

    def _list_format_contributions(self, contribution_type, once=False):
        manager = LilyPondFormatManager
        assert contribution_type in ('override', 'revert')
        result = []
        for attribute_tuple in self._get_attribute_tuples():
            if len(attribute_tuple) == 3:
                context = None
                grob = attribute_tuple[0]
                attribute = attribute_tuple[1]
                value = attribute_tuple[2]
            elif len(attribute_tuple) == 4:
                context = attribute_tuple[0]
                grob = attribute_tuple[1]
                attribute = attribute_tuple[2]
                value = attribute_tuple[3]
            else:
                message = 'invalid attribute tuple: {attribute_tuple!r}.'
                raise ValueError(message)
            if contribution_type == 'override':
                override_string = manager.make_lilypond_override_string(
                    grob,
                    attribute,
                    value,
                    context=context,
                    once=once,
                    )
                result.append(override_string)
            else:
                revert_string = manager.make_lilypond_revert_string(
                    grob,
                    attribute,
                    context=context,
                    )
                result.append(revert_string)
        result.sort()
        return result



from .Leaf import Leaf


class MultimeasureRest(Leaf):
    """
    Multimeasure rest.

    ..  container:: example

        >>> rest = abjad.MultimeasureRest((1, 4))
        >>> abjad.show(rest) # doctest: +SKIP

    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Leaves'

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, *arguments):
        import abjad
        if len(arguments) == 0:
            arguments = ((1, 4),)
        rest = abjad.Rest(*arguments)
        Leaf.__init__(self, rest.written_duration)

    ### PRIVATE METHODS ###

    def _get_body(self):
        """
        Gets list of string representation of body of rest.
        Picked up as format contribution at format-time.
        """
        result = 'R' + str(self._get_formatted_duration())
        return [result]

    def _get_compact_representation(self):
        return f'R{self._get_formatted_duration()}'



"""
Abjad's core component classes: notes, rests, chords, tuplets, measures,
containers and contexts.
"""

from .Component import Component
from .Container import Container
from .GraceContainer import GraceContainer
from .AcciaccaturaContainer import AcciaccaturaContainer
from .AfterGraceContainer import AfterGraceContainer
from .AppoggiaturaContainer import AppoggiaturaContainer
from .Leaf import Leaf
from .Chord import Chord
from .Cluster import Cluster
from .Context import Context
from .Descendants import Descendants
from .NoteHead import NoteHead
from .DrumNoteHead import DrumNoteHead
from .Inspection import Inspection
from .Iteration import Iteration
from .Label import Label
from .LeafMaker import LeafMaker
from .Lineage import Lineage
from .Selection import Selection
from .LogicalTie import LogicalTie
from .Measure import Measure
from .MeasureMaker import MeasureMaker
from .MultimeasureRest import MultimeasureRest
from .Mutation import Mutation
from .Note import Note
from .NoteHeadList import NoteHeadList
from .NoteMaker import NoteMaker
from .Parentage import Parentage
from .Rest import Rest
from .Score import Score
from .Skip import Skip
from .Staff import Staff
from .StaffGroup import StaffGroup
from .Tuplet import Tuplet
from .VerticalMoment import VerticalMoment
from .Voice import Voice



import functools
import typing
from abjad.system.AbjadValueObject import AbjadValueObject
from abjad.markups import Markup
from abjad.utilities.Multiplier import Multiplier


@functools.total_ordering
class BowContactPoint(AbjadValueObject):
    """
    Bow contact point.

    ..  container:: example

        Contact point exactly halfway from frog to tip of bow:

        >>> point = abjad.BowContactPoint((1, 2))
        >>> abjad.f(point)
        abjad.BowContactPoint(
            contact_point=abjad.Multiplier(1, 2),
            )

    ..  container:: example

        Contact point 3/5 of the way from frog to tip of bow:

        >>> point = abjad.BowContactPoint((3, 5))
        >>> abjad.f(point)
        abjad.BowContactPoint(
            contact_point=abjad.Multiplier(3, 5),
            )

    Contact points are measured from frog to tip as a fraction between ``0``
    and ``1``.
    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_contact_point',
        )

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(
        self,
        contact_point: typing.Tuple[int, int] = None
        ) -> None:
        contact_point_ = None
        if contact_point is not None:
            contact_point_ = Multiplier(contact_point)
            assert 0 <= contact_point_ <= 1
        self._contact_point = contact_point_

    ### SPECIAL METHODS ###

    def __lt__(self, argument) -> bool:
        """
        Is true if ``argument`` is a bow contact point and this bow contact
        point is less than ``argument``.

        ..  container:: example

            >>> point_1 = abjad.BowContactPoint((1, 2))
            >>> point_2 = abjad.BowContactPoint((1, 2))
            >>> point_3 = abjad.BowContactPoint((2, 3))

            >>> point_1 < point_1
            False
            >>> point_1 < point_2
            False
            >>> point_1 < point_3
            True

            >>> point_2 < point_1
            False
            >>> point_2 < point_2
            False
            >>> point_2 < point_3
            True

            >>> point_3 < point_1
            False
            >>> point_3 < point_2
            False
            >>> point_3 < point_3
            False

        """
        if isinstance(argument, type(self)):
            self_contact_point = self.contact_point or 0
            argument_contact_point = argument.contact_point or 0
            return self_contact_point < argument_contact_point
        raise TypeError('unorderable types')

    ### PUBLIC PROPERTIES ###

    @property
    def contact_point(self) -> typing.Optional[Multiplier]:
        """
        Gets contact point of bow contact point.

        ..  container:: example

            One quarter of the way from frog to point:

            >>> point = abjad.BowContactPoint((1, 4))
            >>> point.contact_point
            Multiplier(1, 4)

        ..  container:: example

            Three fifths of the way from frog to point:

            >>> point = abjad.BowContactPoint((3, 5))
            >>> point.contact_point
            Multiplier(3, 5)

        """
        return self._contact_point

    @property
    def markup(self) -> Markup:
        r"""
        Gets markup of bow contact point.

        ..  container:: example

            One quarter of the way from frog to point:

            >>> indicator = abjad.BowContactPoint((1, 4))
            >>> print(format(indicator.markup, 'lilypond'))
            \markup {
                \center-align
                    \vcenter
                        \fraction
                            1
                            4
                }
            >>> abjad.show(indicator.markup) # doctest: +SKIP

        ..  container:: example

            Three fifths of the way from frog to point:

            >>> indicator = abjad.BowContactPoint((3, 5))
            >>> print(format(indicator.markup, 'lilypond'))
            \markup {
                \center-align
                    \vcenter
                        \fraction
                            3
                            5
                }
            >>> abjad.show(indicator.markup) # doctest: +SKIP

        """
        if self.contact_point is None:
            contact_point = Multiplier(0, 1)
        else:
            contact_point = self.contact_point
        markup = Markup.fraction(
            contact_point.numerator,
            contact_point.denominator,
            )
        markup = markup.vcenter()
        markup = markup.center_align()
        return markup

    @property
    def tweaks(self) -> None:
        """
        Are not implemented on bow contact point.
        """
        pass



import typing
from abjad.system.AbjadValueObject import AbjadValueObject


class BowPressure(AbjadValueObject):
    """
    Bow pressure indicator.

    ..  container:: example

        >>> bow_pressure = abjad.BowPressure('overpressure')
        >>> abjad.f(bow_pressure)
        abjad.BowPressure(
            pressure='overpressure',
            )

        >>> bow_pressure = abjad.BowPressure('underpressure')
        >>> abjad.f(bow_pressure)
        abjad.BowPressure(
            pressure='underpressure',
            )

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_pressure',
        )

    _parameter = True

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(self, pressure: str = None) -> None:
        self._pressure = pressure

    ### PUBLIC PROPERTIES ###

    @property
    def parameter(self) -> bool:
        """
        Is true.

        ..  container:: example

            >>> abjad.BowPressure('overpressure').parameter
            True

        """
        return self._parameter

    @property
    def pressure(self) -> typing.Optional[str]:
        """
        Gets pressure of indicator.

        ..  container:: example

            Overpressure:

            >>> bow_pressure = abjad.BowPressure('overpressure')
            >>> bow_pressure.pressure
            'overpressure'

        ..  container:: example

            Underpressure:

            >>> bow_pressure = abjad.BowPressure('underpressure')
            >>> bow_pressure.pressure
            'underpressure'

        """
        return self._pressure

    @property
    def tweaks(self) -> None:
        """
        Are not implemented on bow pressure.
        """
        pass



from abjad import markups
from abjad.system.AbjadValueObject import AbjadValueObject


class Multiplication(AbjadValueObject):
    """
    Multiplication operator.

    ..  container:: example

        >>> abjad.Multiplication()
        Multiplication(n=1)

    ..  container:: example

        >>> abjad.Multiplication(n=5)
        Multiplication(n=5)

    Object model of twelve-tone multiplication operator.
    """

    ### CLASS VARIABLES ##

    __slots__ = (
        '_n',
        )

    ### INITIALIZER ###

    def __init__(self, *, n=1):
        self._n = n

    ### SPECIAL METHODS ###

    def __add__(self, operator):
        r"""
        Composes multiplication and `operator`.

        ..  container:: example

            Example segment:

            >>> items = [0, 2, 4, 5]
            >>> segment = abjad.PitchClassSegment(items=items)
            >>> abjad.show(segment) # doctest: +SKIP

            Example operators:

            >>> multiplication = abjad.Multiplication(n=5)
            >>> transposition = abjad.Transposition(n=3)

        ..  container:: example


            Transposition followed by multiplication:

            >>> operator = multiplication + transposition
            >>> str(operator)
            'M5T3'

            >>> segment_ = operator(segment)
            >>> abjad.show(segment_) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment_.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    ef'8
                    cs'8
                    b'8
                    e'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        ..  container:: example

            Same as above because multiplication and transposition commute:

            >>> operator = transposition + multiplication
            >>> str(operator)
            'T3M5'

            >>> segment_ = operator(segment)
            >>> abjad.show(segment_) # doctest: +SKIP

            ..  docs::

                >>> lilypond_file = segment_.__illustrate__()
                >>> abjad.f(lilypond_file[abjad.Voice])
                \new Voice
                {
                    ef'8
                    cs'8
                    b'8
                    e'8
                    \bar "|." %! SCORE1
                    \override Score.BarLine.transparent = ##f
                }

        Returns compound operator.
        """
        import abjad
        return abjad.CompoundOperator._compose_operators(self, operator)

    def __call__(self, argument):
        """
        Calls multiplication on `argument`.

        ..  container:: example

            Multiplies pitch-class:

            >>> multiplication = abjad.Multiplication(n=5)
            >>> pitch_class = abjad.NumberedPitchClass(4)
            >>> multiplication(pitch_class)
            NumberedPitchClass(8)

        ..  container:: example

            Multiplies pitch:

            >>> multiplication = abjad.Multiplication(n=7)
            >>> pitch = abjad.NamedPitch("f'")
            >>> multiplication(pitch)
            NamedPitch("b'''")

        Returns new object with type equal to that of `argument`.
        """
        if hasattr(argument, 'multiply'):
            result = argument.multiply(self.n)
        else:
            message = 'do not know how to multiply: {!r}.'
            message = message.format(argument)
            raise TypeError(message)
        return result

    def __radd__(self, operator):
        """
        Right-addition not defined on multiplication.

        ..  container:: example

            >>> abjad.Multiplication().__radd__(abjad.Multiplication())
            Traceback (most recent call last):
                ...
            NotImplementedError: right-addition not defined on Multiplication.

        Raises not implemented error.
        """
        message = 'right-addition not defined on {}.'
        message = message.format(type(self).__name__)
        raise NotImplementedError(message)

    def __str__(self):
        """
        Gets string representation of operator.

        ..  container:: example

            >>> str(abjad.Multiplication())
            'M1'

        ..  container:: example

            >>> str(abjad.Multiplication(n=5))
            'M5'

        """
        string = 'M{}'
        string = string.format(self.n)
        return string

    ### PRIVATE METHODS ###

    def _get_markup(self, direction=None):
        operator = markups.Markup('M', direction=direction)
        subscript = markups.Markup(self.n).sub()
        markup = markups.Markup.concat([operator, subscript])
        return markup

    def _is_identity_operator(self):
        if self.n == 1:
            return True
        return False

    ### PUBLIC PROPERTIES ###

    @property
    def n(self):
        """
        Gets index of multiplication.

        ..  container:: example

            >>> multiplication = abjad.Multiplication()
            >>> multiplication.n
            1

        ..  container:: example

            >>> multiplication = abjad.Multiplication(n=5)
            >>> multiplication.n
            5

        Set to integer or none.
        """
        return self._n



from abjad.utilities.OrderedDict import OrderedDict
from .ScoreTemplate import ScoreTemplate


class StringOrchestraScoreTemplate(ScoreTemplate):
    r"""
    String orchestra score template.

    ..  container:: example

        >>> template = abjad.StringOrchestraScoreTemplate()
        >>> abjad.show(template) # doctest: +SKIP

        >>> abjad.f(template.__illustrate__()[abjad.Score])
        \context Score = "Score"
        <<
            \tag #'(Violin1 Violin2 Violin3 Violin4 Violin5 Violin6 Viola1 Viola2 Viola3 Viola4 Cello1 Cello2 Cello3 Contrabass1 Contrabass2)
            \context GlobalContext = "GlobalContext"
            {
            }
            \context StaffGroup = "Outer Staff Group"
            <<
                \context ViolinStaffGroup = "Violin Staff Group"
                <<
                    \tag #'Violin1
                    \context StringPerformerStaffGroup = "Violin 1 Staff Group"
                    <<
                        \context BowingStaff = "Violin 1 Bowing Staff"
                        <<
                            \context BowingVoice = "Violin 1 Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Violin 1 Fingering Staff"
                        <<
                            \context FingeringVoice = "Violin 1 Fingering Voice"
                            {
                                \clef "treble" %! ST3
                                s1
                            }
                        >>
                    >>
                    \tag #'Violin2
                    \context StringPerformerStaffGroup = "Violin 2 Staff Group"
                    <<
                        \context BowingStaff = "Violin 2 Bowing Staff"
                        <<
                            \context BowingVoice = "Violin 2 Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Violin 2 Fingering Staff"
                        <<
                            \context FingeringVoice = "Violin 2 Fingering Voice"
                            {
                                \clef "treble" %! ST3
                                s1
                            }
                        >>
                    >>
                    \tag #'Violin3
                    \context StringPerformerStaffGroup = "Violin 3 Staff Group"
                    <<
                        \context BowingStaff = "Violin 3 Bowing Staff"
                        <<
                            \context BowingVoice = "Violin 3 Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Violin 3 Fingering Staff"
                        <<
                            \context FingeringVoice = "Violin 3 Fingering Voice"
                            {
                                \clef "treble" %! ST3
                                s1
                            }
                        >>
                    >>
                    \tag #'Violin4
                    \context StringPerformerStaffGroup = "Violin 4 Staff Group"
                    <<
                        \context BowingStaff = "Violin 4 Bowing Staff"
                        <<
                            \context BowingVoice = "Violin 4 Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Violin 4 Fingering Staff"
                        <<
                            \context FingeringVoice = "Violin 4 Fingering Voice"
                            {
                                \clef "treble" %! ST3
                                s1
                            }
                        >>
                    >>
                    \tag #'Violin5
                    \context StringPerformerStaffGroup = "Violin 5 Staff Group"
                    <<
                        \context BowingStaff = "Violin 5 Bowing Staff"
                        <<
                            \context BowingVoice = "Violin 5 Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Violin 5 Fingering Staff"
                        <<
                            \context FingeringVoice = "Violin 5 Fingering Voice"
                            {
                                \clef "treble" %! ST3
                                s1
                            }
                        >>
                    >>
                    \tag #'Violin6
                    \context StringPerformerStaffGroup = "Violin 6 Staff Group"
                    <<
                        \context BowingStaff = "Violin 6 Bowing Staff"
                        <<
                            \context BowingVoice = "Violin 6 Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Violin 6 Fingering Staff"
                        <<
                            \context FingeringVoice = "Violin 6 Fingering Voice"
                            {
                                \clef "treble" %! ST3
                                s1
                            }
                        >>
                    >>
                >>
                \context ViolaStaffGroup = "Viola Staff Group"
                <<
                    \tag #'Viola1
                    \context StringPerformerStaffGroup = "Viola 1 Staff Group"
                    <<
                        \context BowingStaff = "Viola 1 Bowing Staff"
                        <<
                            \context BowingVoice = "Viola 1 Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Viola 1 Fingering Staff"
                        <<
                            \context FingeringVoice = "Viola 1 Fingering Voice"
                            {
                                \clef "alto" %! ST3
                                s1
                            }
                        >>
                    >>
                    \tag #'Viola2
                    \context StringPerformerStaffGroup = "Viola 2 Staff Group"
                    <<
                        \context BowingStaff = "Viola 2 Bowing Staff"
                        <<
                            \context BowingVoice = "Viola 2 Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Viola 2 Fingering Staff"
                        <<
                            \context FingeringVoice = "Viola 2 Fingering Voice"
                            {
                                \clef "alto" %! ST3
                                s1
                            }
                        >>
                    >>
                    \tag #'Viola3
                    \context StringPerformerStaffGroup = "Viola 3 Staff Group"
                    <<
                        \context BowingStaff = "Viola 3 Bowing Staff"
                        <<
                            \context BowingVoice = "Viola 3 Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Viola 3 Fingering Staff"
                        <<
                            \context FingeringVoice = "Viola 3 Fingering Voice"
                            {
                                \clef "alto" %! ST3
                                s1
                            }
                        >>
                    >>
                    \tag #'Viola4
                    \context StringPerformerStaffGroup = "Viola 4 Staff Group"
                    <<
                        \context BowingStaff = "Viola 4 Bowing Staff"
                        <<
                            \context BowingVoice = "Viola 4 Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Viola 4 Fingering Staff"
                        <<
                            \context FingeringVoice = "Viola 4 Fingering Voice"
                            {
                                \clef "alto" %! ST3
                                s1
                            }
                        >>
                    >>
                >>
                \context CelloStaffGroup = "Cello Staff Group"
                <<
                    \tag #'Cello1
                    \context StringPerformerStaffGroup = "Cello 1 Staff Group"
                    <<
                        \context BowingStaff = "Cello 1 Bowing Staff"
                        <<
                            \context BowingVoice = "Cello 1 Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Cello 1 Fingering Staff"
                        <<
                            \context FingeringVoice = "Cello 1 Fingering Voice"
                            {
                                \clef "bass" %! ST3
                                s1
                            }
                        >>
                    >>
                    \tag #'Cello2
                    \context StringPerformerStaffGroup = "Cello 2 Staff Group"
                    <<
                        \context BowingStaff = "Cello 2 Bowing Staff"
                        <<
                            \context BowingVoice = "Cello 2 Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Cello 2 Fingering Staff"
                        <<
                            \context FingeringVoice = "Cello 2 Fingering Voice"
                            {
                                \clef "bass" %! ST3
                                s1
                            }
                        >>
                    >>
                    \tag #'Cello3
                    \context StringPerformerStaffGroup = "Cello 3 Staff Group"
                    <<
                        \context BowingStaff = "Cello 3 Bowing Staff"
                        <<
                            \context BowingVoice = "Cello 3 Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Cello 3 Fingering Staff"
                        <<
                            \context FingeringVoice = "Cello 3 Fingering Voice"
                            {
                                \clef "bass" %! ST3
                                s1
                            }
                        >>
                    >>
                >>
                \context ContrabassStaffGroup = "Contrabass Staff Group"
                <<
                    \tag #'Contrabass1
                    \context StringPerformerStaffGroup = "Contrabass 1 Staff Group"
                    <<
                        \context BowingStaff = "Contrabass 1 Bowing Staff"
                        <<
                            \context BowingVoice = "Contrabass 1 Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Contrabass 1 Fingering Staff"
                        <<
                            \context FingeringVoice = "Contrabass 1 Fingering Voice"
                            {
                                \clef "bass_8" %! ST3
                                s1
                            }
                        >>
                    >>
                    \tag #'Contrabass2
                    \context StringPerformerStaffGroup = "Contrabass 2 Staff Group"
                    <<
                        \context BowingStaff = "Contrabass 2 Bowing Staff"
                        <<
                            \context BowingVoice = "Contrabass 2 Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Contrabass 2 Fingering Staff"
                        <<
                            \context FingeringVoice = "Contrabass 2 Fingering Voice"
                            {
                                \clef "bass_8" %! ST3
                                s1
                            }
                        >>
                    >>
                >>
            >>
        >>

    ..  container:: example

        As a string quartet:

        >>> template = abjad.StringOrchestraScoreTemplate(
        ...     violin_count=2,
        ...     viola_count=1,
        ...     cello_count=1,
        ...     contrabass_count=0,
        ...     )
        >>> abjad.show(template) # doctest: +SKIP

        >>> abjad.f(template.__illustrate__()[abjad.Score])
        \context Score = "Score"
        <<
            \tag #'(Violin1 Violin2 Viola Cello)
            \context GlobalContext = "GlobalContext"
            {
            }
            \context StaffGroup = "Outer Staff Group"
            <<
                \context ViolinStaffGroup = "Violin Staff Group"
                <<
                    \tag #'Violin1
                    \context StringPerformerStaffGroup = "Violin 1 Staff Group"
                    <<
                        \context BowingStaff = "Violin 1 Bowing Staff"
                        <<
                            \context BowingVoice = "Violin 1 Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Violin 1 Fingering Staff"
                        <<
                            \context FingeringVoice = "Violin 1 Fingering Voice"
                            {
                                \clef "treble" %! ST3
                                s1
                            }
                        >>
                    >>
                    \tag #'Violin2
                    \context StringPerformerStaffGroup = "Violin 2 Staff Group"
                    <<
                        \context BowingStaff = "Violin 2 Bowing Staff"
                        <<
                            \context BowingVoice = "Violin 2 Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Violin 2 Fingering Staff"
                        <<
                            \context FingeringVoice = "Violin 2 Fingering Voice"
                            {
                                \clef "treble" %! ST3
                                s1
                            }
                        >>
                    >>
                >>
                \context ViolaStaffGroup = "Viola Staff Group"
                <<
                    \tag #'Viola
                    \context StringPerformerStaffGroup = "Viola Staff Group"
                    <<
                        \context BowingStaff = "Viola Bowing Staff"
                        <<
                            \context BowingVoice = "Viola Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Viola Fingering Staff"
                        <<
                            \context FingeringVoice = "Viola Fingering Voice"
                            {
                                \clef "alto" %! ST3
                                s1
                            }
                        >>
                    >>
                >>
                \context CelloStaffGroup = "Cello Staff Group"
                <<
                    \tag #'Cello
                    \context StringPerformerStaffGroup = "Cello Staff Group"
                    <<
                        \context BowingStaff = "Cello Bowing Staff"
                        <<
                            \context BowingVoice = "Cello Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Cello Fingering Staff"
                        <<
                            \context FingeringVoice = "Cello Fingering Voice"
                            {
                                \clef "bass" %! ST3
                                s1
                            }
                        >>
                    >>
                >>
            >>
        >>

    ..  container:: example

        As a cello solo:

        >>> template = abjad.StringOrchestraScoreTemplate(
        ...     violin_count=0,
        ...     viola_count=0,
        ...     cello_count=1,
        ...     contrabass_count=0,
        ...     )
        >>> abjad.show(template) # doctest: +SKIP

        >>> abjad.f(template.__illustrate__()[abjad.Score])
        \context Score = "Score"
        <<
            \tag #'(Cello)
            \context GlobalContext = "GlobalContext"
            {
            }
            \context StaffGroup = "Outer Staff Group"
            <<
                \context CelloStaffGroup = "Cello Staff Group"
                <<
                    \tag #'Cello
                    \context StringPerformerStaffGroup = "Cello Staff Group"
                    <<
                        \context BowingStaff = "Cello Bowing Staff"
                        <<
                            \context BowingVoice = "Cello Bowing Voice"
                            {
                                s1
                            }
                        >>
                        \context FingeringStaff = "Cello Fingering Staff"
                        <<
                            \context FingeringVoice = "Cello Fingering Voice"
                            {
                                \clef "bass" %! ST3
                                s1
                            }
                        >>
                    >>
                >>
            >>
        >>

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_cello_count',
        '_contrabass_count',
        '_split_hands',
        '_use_percussion_clefs',
        '_viola_count',
        '_violin_count',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        violin_count=6,
        viola_count=4,
        cello_count=3,
        contrabass_count=2,
        split_hands=True,
        use_percussion_clefs=False,
        ):
        assert 0 <= violin_count
        assert 0 <= viola_count
        assert 0 <= cello_count
        assert 0 <= contrabass_count
        super().__init__()
        self._violin_count = int(violin_count)
        self._viola_count = int(viola_count)
        self._cello_count = int(cello_count)
        self._contrabass_count = int(contrabass_count)
        self._split_hands = bool(split_hands)
        self._use_percussion_clefs = bool(use_percussion_clefs)

    ### SPECIAL METHODS ###

    def __call__(self):
        """
        Calls string orchestra template.

        Returns score.
        """
        import abjad

        ### TAGS ###

        tag_names = []

        ### SCORE ###

        staff_group = abjad.StaffGroup(
            name='Outer Staff Group',
            )

        score = abjad.Score(
            [staff_group],
            name='Score',
            )

        ### VIOLINS ###

        if self.violin_count:
            clef_name = 'treble'
            if self.use_percussion_clefs:
                clef_name = 'percussion'
            instrument = abjad.Violin()
            instrument_count = self.violin_count
            instrument_staff_group, instrument_tag_names = \
                self._make_instrument_staff_group(
                    clef_name=clef_name,
                    count=instrument_count,
                    instrument=instrument,
                    )
            staff_group.append(instrument_staff_group)
            tag_names.extend(instrument_tag_names)

        ### VIOLAS ###

        if self.viola_count:
            clef_name = 'alto'
            if self.use_percussion_clefs:
                clef_name = 'percussion'
            instrument = abjad.Viola()
            instrument_count = self.viola_count
            instrument_staff_group, instrument_tag_names = \
                self._make_instrument_staff_group(
                    clef_name=clef_name,
                    count=instrument_count,
                    instrument=instrument,
                    )
            staff_group.append(instrument_staff_group)
            tag_names.extend(instrument_tag_names)

        ### CELLOS ###

        if self.cello_count:
            clef_name = 'bass'
            if self.use_percussion_clefs:
                clef_name = 'percussion'
            instrument = abjad.Cello()
            instrument_count = self.cello_count
            instrument_staff_group, instrument_tag_names = \
                self._make_instrument_staff_group(
                    clef_name=clef_name,
                    count=instrument_count,
                    instrument=instrument,
                    )
            staff_group.append(instrument_staff_group)
            tag_names.extend(instrument_tag_names)

        ### BASSES ###

        if self.contrabass_count:
            clef_name = 'bass_8'
            if self.use_percussion_clefs:
                clef_name = 'percussion'
            instrument = abjad.Contrabass()
            instrument_count = self.contrabass_count
            instrument_staff_group, instrument_tag_names = \
                self._make_instrument_staff_group(
                    clef_name=clef_name,
                    count=instrument_count,
                    instrument=instrument,
                    )
            staff_group.append(instrument_staff_group)
            tag_names.extend(instrument_tag_names)

        ### TIME SIGNATURE CONTEXT ###

        global_context = abjad.Context(
            name='GlobalContext',
            lilypond_type='GlobalContext',
            )
        instrument_tags = ' '.join(tag_names)
        tag_string = r"\tag #'({})".format(instrument_tags)
        tag_command = abjad.LilyPondLiteral(tag_string, 'before')
        abjad.attach(tag_command, global_context)
        score.insert(0, global_context)
        return score

    ### PRIVATE METHODS ###

    def _make_instrument_staff_group(
        self,
        clef_name=None,
        count=None,
        instrument=None,
        ):
        import abjad
        name = instrument.name.title()
        instrument_staff_group = abjad.StaffGroup(
            lilypond_type='{}StaffGroup'.format(name),
            name='{} Staff Group'.format(name),
            )
        tag_names = []
        if count == 1:
            performer_staff_group, tag_name = \
                self._make_performer_staff_group(
                    clef_name=clef_name,
                    instrument=instrument,
                    number=None,
                    )
            instrument_staff_group.append(performer_staff_group)
            tag_names.append(tag_name)
        else:
            for i in range(1, count + 1):
                performer_staff_group, tag_name = \
                    self._make_performer_staff_group(
                        clef_name=clef_name,
                        instrument=instrument,
                        number=i,
                        )
                instrument_staff_group.append(performer_staff_group)
                tag_names.append(tag_name)
        return instrument_staff_group, tag_names

    def _make_performer_staff_group(
        self,
        clef_name=None,
        instrument=None,
        number=None,
        ):
        import abjad
        if number is not None:
            name = '{} {}'.format(
                instrument.name.title(),
                number,
                )
        else:
            name = instrument.name.title()
        pitch_range = instrument.pitch_range
        staff_group = abjad.StaffGroup(
            lilypond_type='StringPerformerStaffGroup',
            name='{} Staff Group'.format(name),
            )
        tag_name = name.replace(' ', '')
        tag_string = r"\tag #'{}".format(tag_name)
        tag_command = abjad.LilyPondLiteral(tag_string, 'before')
        abjad.attach(tag_command, staff_group)
        if self.split_hands:
            lh_voice = abjad.Voice(
                [],
                lilypond_type='FingeringVoice',
                name='{} Fingering Voice'.format(name),
                )
            abbreviation = lh_voice.name.lower().replace(' ', '_')
            self.voice_abbreviations[abbreviation] = lh_voice.name
            lh_staff = abjad.Staff(
                [
                    lh_voice
                    ],
                lilypond_type='FingeringStaff',
                name='{} Fingering Staff'.format(name),
                )
            lh_staff.is_simultaneous = True
            abjad.annotate(lh_staff, 'pitch_range', pitch_range)
            abjad.annotate(lh_staff, 'default_clef', abjad.Clef(clef_name))
            rh_voice = abjad.Voice(
                [],
                lilypond_type='BowingVoice',
                name='{} Bowing Voice'.format(name),
                )
            abbreviation = rh_voice.name.lower().replace(' ', '_')
            self.voice_abbreviations[abbreviation] = rh_voice.name
            rh_staff = abjad.Staff(
                [
                    rh_voice
                    ],
                lilypond_type='BowingStaff',
                name='{} Bowing Staff'.format(name),
                )
            rh_staff.is_simultaneous = True
            staff_group.extend([rh_staff, lh_staff])
        else:
            lh_voice = abjad.Voice(
                [],
                lilypond_type='FingeringVoice',
                name='{} Voice'.format(name),
                )
            lh_staff = abjad.Staff(
                [
                    lh_voice
                    ],
                lilypond_type='FingeringStaff',
                name='{} Staff'.format(name),
                )
            lh_staff.is_simultaneous = True
            abjad.annotate(lh_staff, 'pitch_range', pitch_range)
            abjad.annotate(lh_staff, 'default_clef', abjad.Clef(clef_name))
            staff_group.append(lh_staff)
        return staff_group, tag_name

    ### PUBLIC PROPERTIES ###

    @property
    def cello_count(self):
        """
        Number of cellos in string orchestra.

        Returns nonnegative integer.
        """
        return self._cello_count

    @property
    def contrabass_count(self):
        """
        Number of contrabasses in string orchestra.

        Returns nonnegative integer.
        """
        return self._contrabass_count

    @property
    def split_hands(self):
        """
        Is true if each performer's hand receives a separate staff.
        """
        return self._split_hands

    @property
    def use_percussion_clefs(self):
        """
        Is true if each staff should use a percussion clef rather than the
        normal clef for that instrument.
        """
        return self._use_percussion_clefs

    @property
    def viola_count(self):
        """
        Number of violas in string orcestra.

        Returns nonnegative integer.
        """
        return self._viola_count

    @property
    def violin_count(self):
        """
        Number of violins in string orchestra.

        Returns nonnegative integer.
        """
        return self._violin_count



import numbers


def is_nonnegative_integer(argument):
    """
    Is true when ``argument`` equals a nonnegative integer.

    ..  container:: example

        >>> abjad.mathtools.is_nonnegative_integer(99)
        True

        >>> abjad.mathtools.is_nonnegative_integer(0)
        True

        >>> abjad.mathtools.is_nonnegative_integer(-1)
        False

    Returns true or false.
    """
    if isinstance(argument, numbers.Number):
        if argument == int(argument):
            if 0 <= argument:
                return True
    return False



from abjad import utilities
from .Music import Music


class ContextSpeccedMusic(Music):
    """
    Abjad model of the LilyPond AST context-specced music node.
    """

    ### CLASS VARIABLES ###

    __slots__ = (
        #'context',
        'lilypond_type',
        'music',
        'optional_id',
        'optional_context_mod',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        lilypond_type=None,
        optional_id=None,
        optional_context_mod=None,
        music=None,
        ):
        from abjad import parser as abjad_parser
        lilypond_type = lilypond_type or ''
        music = music or abjad_parser.SequentialMusic()
        assert utilities.String.is_string(lilypond_type)
        assert isinstance(music, Music)
        self.lilypond_type = lilypond_type
        self.optional_id = optional_id
        self.optional_context_mod = optional_context_mod
        self.music = music

    ### PUBLIC METHODS ###

    def construct(self):
        """
        Constructs context.

        Returns context.
        """
        if self.lilypond_type in self.known_contexts:
            context = known_contexts[self.lilypond_type]([])
        else:
            message = 'context type not supported: {}.'
            message = message.format(self.lilypond_type)
            raise Exception(message)

        if self.optional_id is not None:
            context.name = self.optional_id

        if self.optional_context_mod is not None:
            for x in self.optional_context_mod:
                print(x)
            # TODO: implement context modifications on contexts
            pass

        if isinstance(self.music, abjad_parser.SimultaneousMusic):
            context.is_simultaneous = True
        context.extend(music.construct())

        return context

    ### PUBLIC PROPERTIES ###

    @property
    def known_contexts(self):
        """
        Known contexts.

        Returns dictionary.
        """
        from abjad import core
        return {
            'ChoirStaff': core.StaffGroup,
            'GrandStaff': core.StaffGroup,
            'PianoStaff': core.StaffGroup,
            'Score': core.Score,
            'Staff': core.Staff,
            'StaffGroup': core.StaffGroup,
            'Voice': core.Voice,
        }



from abjad import mathtools
from abjad.top.new import new
from abjad.utilities.Multiplier import Multiplier
from .Segment import Segment


class IntervalSegment(Segment):
    """
    Interval segment.

    ..  container:: example

        Initializes from string:

        >>> intervals = 'm2 M10 -aug4 P5'
        >>> abjad.IntervalSegment(intervals)
        IntervalSegment(['+m2', '+M10', '-A4', '+P5'])

    ..  container:: example

        Initializes from pitch segment:

        >>> pitch_segment = abjad.PitchSegment("c d e f g a b c'")
        >>> abjad.IntervalSegment(pitch_segment)
        IntervalSegment(['+M2', '+M2', '+m2', '+M2', '+M2', '+M2', '+m2'])

    """

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, items=None, item_class=None):
        import abjad
        if isinstance(items, abjad.PitchSegment):
            intervals = []
            for one, two in abjad.sequence(items).nwise():
                intervals.append(one - two)
            items = intervals
        Segment.__init__(
            self,
            items=items,
            item_class=item_class,
            )

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        import abjad
        return abjad.NamedInterval

    @property
    def _numbered_item_class(self):
        import abjad
        return abjad.NumberedInterval

    @property
    def _parent_item_class(self):
        import abjad
        return abjad.Interval

    ### PUBLIC PROPERTIES ###

    @property
    def slope(self):
        """
        Gets slope of interval segment.

        ..  container:: example

            The slope of a interval segment is the sum of its
            intervals divided by its length:

            >>> abjad.IntervalSegment([1, 2]).slope
            Multiplier(3, 2)

        Returns multiplier.
        """
        result = sum([x.number for x in self]) / len(self)
        return Multiplier.from_float(result)

    @property
    def spread(self):
        """
        Gets spread of interval segment.

        ..  container:: example

            The maximum interval spanned by any combination of
            the intervals within a numbered interval segment.

            >>> abjad.IntervalSegment([1, 2, -3, 1, -2, 1]).spread
            NumberedInterval(4)

            >>> abjad.IntervalSegment([1, 1, 1, 2, -3, -2]).spread
            NumberedInterval(5)

        Returns numbered interval.
        """
        import abjad
        current = maximum = minimum = 0
        for x in self:
            current += float(x.number)
            if maximum < current:
                maximum = current
            if current < minimum:
                minimum = current
        return abjad.NumberedInterval(maximum - minimum)

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        class_,
        selection,
        item_class=None,
        ):
        """
        Makes interval segment from component `selection`.

        ..  container:: example

            >>> staff = abjad.Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")
            >>> abjad.IntervalSegment.from_selection(
            ...     staff,
            ...     item_class=abjad.NumberedInterval,
            ...     )
            IntervalSegment([2, 2, 1, 2, 2, 2, 1])

        Returns interval segment.
        """
        import abjad
        pitch_segment = abjad.PitchSegment.from_selection(selection)
        pitches = [_ for _ in pitch_segment]
        intervals = (-x for x in mathtools.difference_series(pitches))
        return class_(
            items=intervals,
            item_class=item_class,
            )

    def has_duplicates(self):
        """
        Is true if segment has duplicate items.

        ..  container:: example

            >>> intervals = 'm2 M3 -aug4 m2 P5'
            >>> segment = abjad.IntervalSegment(intervals)
            >>> segment.has_duplicates()
            True

            >>> intervals = 'M3 -aug4 m2 P5'
            >>> segment = abjad.IntervalSegment(intervals)
            >>> segment.has_duplicates()
            False

        Returns true or false.
        """
        import abjad
        return len(abjad.IntervalSet(self)) < len(self)

    def rotate(self, n=0):
        """
        Rotates interval segment by index `n`.

        Returns new interval segment.
        """
        return new(self, self[-n:] + self[:-n])



import copy


class LilyPondNameManager(object):
    """
    LilyPond name manager.

    Base class from which grob, setting and tweak managers inherit.
    """

    ### SPECIAL METHODS ###

    def __eq__(self, argument) -> bool:
        """
        Is true when ``argument`` is a LilyPond name manager with attribute
        pairs equal to those of this LilyPond name manager.
        """
        if isinstance(argument, type(self)):
            attribute_pairs_1 = self._get_attribute_pairs()
            attribute_pairs_2 = argument._get_attribute_pairs()
            return attribute_pairs_1 == attribute_pairs_2
        return False

    def __getstate__(self) -> dict:
        """
        Gets object state.
        """
        return copy.deepcopy(vars(self))

    def __hash__(self) -> int:
        """
        Hashes LilyPond name manager.

        Redefined in tandem with __eq__.
        """
        return super().__hash__()

    def __repr__(self) -> str:
        """
        Gets interpreter representation of LilyPond name manager.
        """
        body_string = ''
        pairs = self._get_attribute_pairs()
        pairs = [str(_) for _ in pairs]
        body_string = ', '.join(pairs)
        return f'{type(self).__name__}({body_string})'

    def __setstate__(self, state) -> None:
        """
        Sets object state.
        """
        for key, value in state.items():
            self.__dict__[key] = value

    ### PRIVATE METHODS ###

    def _get_attribute_pairs(self):
        return list(sorted(vars(self).items()))



def sequence(items=None, **keywords):
    r"""
    Makes sequence or sequence expression.

    ..  container:: example

        ..  container:: example

            Makes sequence:

            >>> abjad.sequence([1, 2, [3, [4]], 5])
            Sequence([1, 2, [3, [4]], 5])

        ..  container:: example expression

            Makes sequence expression:

            >>> expression = abjad.sequence()
            >>> expression([1, 2, [3, [4]], 5])
            Sequence([1, 2, [3, [4]], 5])

    ..  container:: example

        Flattens, reverses and slices sequence:

        ..  container:: example

            >>> sequence_ = abjad.sequence([1, 2, [3, [4]], 5])
            >>> sequence_
            Sequence([1, 2, [3, [4]], 5])

            >>> sequence_ = sequence_.flatten(depth=-1)
            >>> sequence_
            Sequence([1, 2, 3, 4, 5])

            >>> sequence_ = sequence_.reverse()
            >>> sequence_
            Sequence([5, 4, 3, 2, 1])

            >>> sequence_ = sequence_[-3:]
            >>> sequence_
            Sequence([3, 2, 1])

        ..  container:: example expression

            >>> expression = abjad.sequence()
            >>> expression = expression.flatten(depth=-1)
            >>> expression = expression.reverse()
            >>> expression = expression[-3:]
            >>> expression([1, 2, [3, [4]], 5])
            Sequence([3, 2, 1])

    Returns sequence when ``items`` is not none.

    Returns sequence expression when ``items`` is none.
    """
    import abjad
    if items is not None:
        return abjad.Sequence(items=items, **keywords)
    else:
        expression = abjad.Expression()
        expression = expression.sequence(**keywords)
        return expression



lilypond_version = "2.19.24"

engravers = {
    "Accidental_engraver": {
        "grobs_created": set([
            "Accidental",
            "AccidentalCautionary",
            "AccidentalPlacement",
            "AccidentalSuggestion",
            ]),
        "properties_read": set([
            "accidentalGrouping",
            "autoAccidentals",
            "autoCautionaries",
            "extraNatural",
            "harmonicAccidentals",
            "internalBarNumber",
            "keyAlterations",
            "localAlterations",
            ]),
        "properties_written": set([
            "localAlterations",
            ]),
    },
    "Ambitus_engraver": {
        "grobs_created": set([
            "AccidentalPlacement",
            "Ambitus",
            "AmbitusAccidental",
            "AmbitusLine",
            "AmbitusNoteHead",
            ]),
        "properties_read": set([
            "keyAlterations",
            "middleCClefPosition",
            "middleCOffset",
            ]),
        "properties_written": set([]),
    },
    "Arpeggio_engraver": {
        "grobs_created": set([
            "Arpeggio",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Auto_beam_engraver": {
        "grobs_created": set([
            "Beam",
            ]),
        "properties_read": set([
            "autoBeaming",
            "baseMoment",
            "beamExceptions",
            "beamHalfMeasure",
            "beatStructure",
            "subdivideBeams",
            ]),
        "properties_written": set([]),
    },
    "Axis_group_engraver": {
        "grobs_created": set([
            "VerticalAxisGroup",
            ]),
        "properties_read": set([
            "currentCommandColumn",
            "hasAxisGroup",
            "keepAliveInterfaces",
            ]),
        "properties_written": set([
            "hasAxisGroup",
            ]),
    },
    "Balloon_engraver": {
        "grobs_created": set([
            "BalloonTextItem",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Bar_engraver": {
        "grobs_created": set([
            "BarLine",
            ]),
        "properties_read": set([
            "whichBar",
            ]),
        "properties_written": set([
            "forbidBreak",
            ]),
    },
    "Bar_number_engraver": {
        "grobs_created": set([
            "BarNumber",
            ]),
        "properties_read": set([
            "alternativeNumberingStyle",
            "barNumberFormatter",
            "barNumberVisibility",
            "currentBarNumber",
            "stavesFound",
            "whichBar",
            ]),
        "properties_written": set([
            "currentBarNumber",
            ]),
    },
    "Beam_collision_engraver": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Beam_engraver": {
        "grobs_created": set([
            "Beam",
            ]),
        "properties_read": set([
            "baseMoment",
            "beamMelismaBusy",
            "beatStructure",
            "subdivideBeams",
            ]),
        "properties_written": set([
            "forbidBreak",
            ]),
    },
    "Beam_performer": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Bend_engraver": {
        "grobs_created": set([
            "BendAfter",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Break_align_engraver": {
        "grobs_created": set([
            "BreakAlignGroup",
            "BreakAlignment",
            "LeftEdge",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Breathing_sign_engraver": {
        "grobs_created": set([
            "BreathingSign",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Chord_name_engraver": {
        "grobs_created": set([
            "ChordName",
            ]),
        "properties_read": set([
            "chordChanges",
            "chordNameExceptions",
            "chordNameExceptions",
            "chordNameFunction",
            "chordNoteNamer",
            "chordRootNamer",
            "lastChord",
            "majorSevenSymbol",
            "noChordSymbol",
            ]),
        "properties_written": set([
            "lastChord",
            ]),
    },
    "Chord_tremolo_engraver": {
        "grobs_created": set([
            "Beam",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Clef_engraver": {
        "grobs_created": set([
            "Clef",
            "ClefModifier",
            ]),
        "properties_read": set([
            "clefGlyph",
            "clefPosition",
            "clefTransposition",
            "clefTranspositionStyle",
            "explicitClefVisibility",
            "forceClef",
            ]),
        "properties_written": set([]),
    },
    "Cluster_spanner_engraver": {
        "grobs_created": set([
            "ClusterSpanner",
            "ClusterSpannerBeacon",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Collision_engraver": {
        "grobs_created": set([
            "NoteCollision",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Completion_heads_engraver": {
        "grobs_created": set([
            "NoteHead",
            "Tie",
            "TieColumn",
            ]),
        "properties_read": set([
            "completionFactor",
            "completionUnit",
            "measureLength",
            "measurePosition",
            "middleCPosition",
            "timing",
            ]),
        "properties_written": set([
            "completionBusy",
            ]),
    },
    "Completion_rest_engraver": {
        "grobs_created": set([
            "Rest",
            ]),
        "properties_read": set([
            "completionFactor",
            "completionUnit",
            "measureLength",
            "measurePosition",
            "middleCPosition",
            ]),
        "properties_written": set([
            "restCompletionBusy",
            ]),
    },
    "Concurrent_hairpin_engraver": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Control_track_performer": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Cue_clef_engraver": {
        "grobs_created": set([
            "ClefModifier",
            "CueClef",
            "CueEndClef",
            ]),
        "properties_read": set([
            "clefTransposition",
            "cueClefGlyph",
            "cueClefPosition",
            "cueClefTransposition",
            "cueClefTranspositionStyle",
            "explicitCueClefVisibility",
            "middleCCuePosition",
            ]),
        "properties_written": set([]),
    },
    "Custos_engraver": {
        "grobs_created": set([
            "Custos",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Default_bar_line_engraver": {
        "grobs_created": set([]),
        "properties_read": set([
            "automaticBars",
            "barAlways",
            "defaultBarType",
            "measureLength",
            "measurePosition",
            "timing",
            "whichBar",
            ]),
        "properties_written": set([]),
    },
    "Dot_column_engraver": {
        "grobs_created": set([
            "DotColumn",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Dots_engraver": {
        "grobs_created": set([
            "Dots",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Double_percent_repeat_engraver": {
        "grobs_created": set([
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            ]),
        "properties_read": set([
            "countPercentRepeats",
            "measureLength",
            "repeatCountVisibility",
            ]),
        "properties_written": set([
            "forbidBreak",
            ]),
    },
    "Drum_note_performer": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Drum_notes_engraver": {
        "grobs_created": set([
            "NoteHead",
            "Script",
            ]),
        "properties_read": set([
            "drumStyleTable",
            ]),
        "properties_written": set([]),
    },
    "Dynamic_align_engraver": {
        "grobs_created": set([
            "DynamicLineSpanner",
            ]),
        "properties_read": set([
            "currentMusicalColumn",
            ]),
        "properties_written": set([]),
    },
    "Dynamic_engraver": {
        "grobs_created": set([
            "DynamicText",
            "DynamicTextSpanner",
            "Hairpin",
            ]),
        "properties_read": set([
            "crescendoSpanner",
            "crescendoText",
            "currentMusicalColumn",
            "decrescendoSpanner",
            "decrescendoText",
            ]),
        "properties_written": set([]),
    },
    "Dynamic_performer": {
        "grobs_created": set([]),
        "properties_read": set([
            "dynamicAbsoluteVolumeFunction",
            "instrumentEqualizer",
            "midiInstrument",
            "midiMaximumVolume",
            "midiMinimumVolume",
            ]),
        "properties_written": set([]),
    },
    "Engraver": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Episema_engraver": {
        "grobs_created": set([
            "Episema",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Extender_engraver": {
        "grobs_created": set([
            "LyricExtender",
            ]),
        "properties_read": set([
            "extendersOverRests",
            ]),
        "properties_written": set([]),
    },
    "Figured_bass_engraver": {
        "grobs_created": set([
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
            ]),
        "properties_read": set([
            "figuredBassAlterationDirection",
            "figuredBassCenterContinuations",
            "figuredBassFormatter",
            "ignoreFiguredBassRest",
            "implicitBassFigures",
            "useBassFigureExtenders",
            ]),
        "properties_written": set([]),
    },
    "Figured_bass_position_engraver": {
        "grobs_created": set([
            "BassFigureAlignmentPositioning",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Fingering_column_engraver": {
        "grobs_created": set([
            "FingeringColumn",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Fingering_engraver": {
        "grobs_created": set([
            "Fingering",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Font_size_engraver": {
        "grobs_created": set([]),
        "properties_read": set([
            "fontSize",
            ]),
        "properties_written": set([]),
    },
    "Footnote_engraver": {
        "grobs_created": set([
            "FootnoteItem",
            "FootnoteSpanner",
            ]),
        "properties_read": set([
            "currentMusicalColumn",
            ]),
        "properties_written": set([]),
    },
    "Forbid_line_break_engraver": {
        "grobs_created": set([]),
        "properties_read": set([
            "busyGrobs",
            ]),
        "properties_written": set([
            "forbidBreak",
            ]),
    },
    "Fretboard_engraver": {
        "grobs_created": set([
            "FretBoard",
            ]),
        "properties_read": set([
            "chordChanges",
            "defaultStrings",
            "highStringOne",
            "maximumFretStretch",
            "minimumFret",
            "noteToFretFunction",
            "predefinedDiagramTable",
            "stringTunings",
            "tablatureFormat",
            ]),
        "properties_written": set([]),
    },
    "Glissando_engraver": {
        "grobs_created": set([
            "Glissando",
            ]),
        "properties_read": set([
            "glissandoMap",
            ]),
        "properties_written": set([]),
    },
    "Grace_auto_beam_engraver": {
        "grobs_created": set([
            "Beam",
            ]),
        "properties_read": set([
            "autoBeaming",
            ]),
        "properties_written": set([]),
    },
    "Grace_beam_engraver": {
        "grobs_created": set([
            "Beam",
            ]),
        "properties_read": set([
            "baseMoment",
            "beamMelismaBusy",
            "beatStructure",
            "subdivideBeams",
            ]),
        "properties_written": set([]),
    },
    "Grace_engraver": {
        "grobs_created": set([]),
        "properties_read": set([
            "graceSettings",
            ]),
        "properties_written": set([]),
    },
    "Grace_spacing_engraver": {
        "grobs_created": set([
            "GraceSpacing",
            ]),
        "properties_read": set([
            "currentMusicalColumn",
            ]),
        "properties_written": set([]),
    },
    "Grid_line_span_engraver": {
        "grobs_created": set([
            "GridLine",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Grid_point_engraver": {
        "grobs_created": set([
            "GridPoint",
            ]),
        "properties_read": set([
            "gridInterval",
            ]),
        "properties_written": set([]),
    },
    "Grob_pq_engraver": {
        "grobs_created": set([]),
        "properties_read": set([
            "busyGrobs",
            ]),
        "properties_written": set([
            "busyGrobs",
            ]),
    },
    "Horizontal_bracket_engraver": {
        "grobs_created": set([
            "HorizontalBracket",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Hyphen_engraver": {
        "grobs_created": set([
            "LyricHyphen",
            "LyricSpace",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Instrument_name_engraver": {
        "grobs_created": set([
            "InstrumentName",
            ]),
        "properties_read": set([
            "currentCommandColumn",
            "instrumentName",
            "shortInstrumentName",
            "shortVocalName",
            "vocalName",
            ]),
        "properties_written": set([]),
    },
    "Instrument_switch_engraver": {
        "grobs_created": set([
            "InstrumentSwitch",
            ]),
        "properties_read": set([
            "instrumentCueName",
            ]),
        "properties_written": set([]),
    },
    "Keep_alive_together_engraver": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Key_engraver": {
        "grobs_created": set([
            "KeyCancellation",
            "KeySignature",
            ]),
        "properties_read": set([
            "createKeyOnClefChange",
            "explicitKeySignatureVisibility",
            "extraNatural",
            "keyAlterationOrder",
            "keyAlterations",
            "lastKeyAlterations",
            "middleCClefPosition",
            "printKeyCancellation",
            ]),
        "properties_written": set([
            "keyAlterations",
            "lastKeyAlterations",
            "tonic",
            ]),
    },
    "Key_performer": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Kievan_ligature_engraver": {
        "grobs_created": set([
            "KievanLigature",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Laissez_vibrer_engraver": {
        "grobs_created": set([
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Ledger_line_engraver": {
        "grobs_created": set([
            "LedgerLineSpanner",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Ligature_bracket_engraver": {
        "grobs_created": set([
            "LigatureBracket",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Lyric_engraver": {
        "grobs_created": set([
            "LyricText",
            ]),
        "properties_read": set([
            "ignoreMelismata",
            "lyricMelismaAlignment",
            "searchForVoice",
            ]),
        "properties_written": set([]),
    },
    "Lyric_performer": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Mark_engraver": {
        "grobs_created": set([
            "RehearsalMark",
            ]),
        "properties_read": set([
            "markFormatter",
            "rehearsalMark",
            "stavesFound",
            ]),
        "properties_written": set([]),
    },
    "Measure_grouping_engraver": {
        "grobs_created": set([
            "MeasureGrouping",
            ]),
        "properties_read": set([
            "baseMoment",
            "beatStructure",
            "currentMusicalColumn",
            "measurePosition",
            ]),
        "properties_written": set([]),
    },
    "Melody_engraver": {
        "grobs_created": set([
            "MelodyItem",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Mensural_ligature_engraver": {
        "grobs_created": set([
            "MensuralLigature",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Metronome_mark_engraver": {
        "grobs_created": set([
            "MetronomeMark",
            ]),
        "properties_read": set([
            "currentCommandColumn",
            "currentMusicalColumn",
            "metronomeMarkFormatter",
            "stavesFound",
            "tempoHideNote",
            ]),
        "properties_written": set([]),
    },
    "Midi_control_function_performer": {
        "grobs_created": set([]),
        "properties_read": set([
            "midiBalance",
            "midiChorusLevel",
            "midiExpression",
            "midiPanPosition",
            "midiReverbLevel",
            ]),
        "properties_written": set([]),
    },
    "Multi_measure_rest_engraver": {
        "grobs_created": set([
            "MultiMeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            ]),
        "properties_read": set([
            "currentCommandColumn",
            "internalBarNumber",
            "measurePosition",
            "restNumberThreshold",
            ]),
        "properties_written": set([]),
    },
    "New_fingering_engraver": {
        "grobs_created": set([
            "Fingering",
            "Script",
            "StringNumber",
            "StrokeFinger",
            ]),
        "properties_read": set([
            "fingeringOrientations",
            "harmonicDots",
            "stringNumberOrientations",
            "strokeFingerOrientations",
            ]),
        "properties_written": set([]),
    },
    "Note_head_line_engraver": {
        "grobs_created": set([
            "VoiceFollower",
            ]),
        "properties_read": set([
            "followVoice",
            ]),
        "properties_written": set([]),
    },
    "Note_heads_engraver": {
        "grobs_created": set([
            "NoteHead",
            ]),
        "properties_read": set([
            "middleCPosition",
            "staffLineLayoutFunction",
            ]),
        "properties_written": set([]),
    },
    "Note_name_engraver": {
        "grobs_created": set([
            "NoteName",
            ]),
        "properties_read": set([
            "printOctaveNames",
            ]),
        "properties_written": set([]),
    },
    "Note_performer": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Note_spacing_engraver": {
        "grobs_created": set([
            "NoteSpacing",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Ottava_spanner_engraver": {
        "grobs_created": set([
            "OttavaBracket",
            ]),
        "properties_read": set([
            "currentMusicalColumn",
            "middleCOffset",
            "ottavation",
            ]),
        "properties_written": set([]),
    },
    "Output_property_engraver": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Page_turn_engraver": {
        "grobs_created": set([]),
        "properties_read": set([
            "minimumPageTurnLength",
            "minimumRepeatLengthForPageTurn",
            ]),
        "properties_written": set([]),
    },
    "Paper_column_engraver": {
        "grobs_created": set([
            "NonMusicalPaperColumn",
            "PaperColumn",
            ]),
        "properties_read": set([
            "forbidBreak",
            ]),
        "properties_written": set([
            "currentCommandColumn",
            "currentMusicalColumn",
            "forbidBreak",
            ]),
    },
    "Parenthesis_engraver": {
        "grobs_created": set([
            "ParenthesesItem",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Part_combine_engraver": {
        "grobs_created": set([
            "CombineTextScript",
            ]),
        "properties_read": set([
            "aDueText",
            "partCombineTextsOnNote",
            "printPartCombineTexts",
            "soloIIText",
            "soloText",
            ]),
        "properties_written": set([]),
    },
    "Percent_repeat_engraver": {
        "grobs_created": set([
            "PercentRepeat",
            "PercentRepeatCounter",
            ]),
        "properties_read": set([
            "countPercentRepeats",
            "currentCommandColumn",
            "repeatCountVisibility",
            ]),
        "properties_written": set([]),
    },
    "Phrasing_slur_engraver": {
        "grobs_created": set([
            "PhrasingSlur",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Piano_pedal_align_engraver": {
        "grobs_created": set([
            "SostenutoPedalLineSpanner",
            "SustainPedalLineSpanner",
            "UnaCordaPedalLineSpanner",
            ]),
        "properties_read": set([
            "currentCommandColumn",
            ]),
        "properties_written": set([]),
    },
    "Piano_pedal_engraver": {
        "grobs_created": set([
            "PianoPedalBracket",
            "SostenutoPedal",
            "SustainPedal",
            "UnaCordaPedal",
            ]),
        "properties_read": set([
            "currentCommandColumn",
            "pedalSostenutoStrings",
            "pedalSostenutoStyle",
            "pedalSustainStrings",
            "pedalSustainStyle",
            "pedalUnaCordaStrings",
            "pedalUnaCordaStyle",
            ]),
        "properties_written": set([]),
    },
    "Piano_pedal_performer": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Pitch_squash_engraver": {
        "grobs_created": set([]),
        "properties_read": set([
            "squashedPosition",
            ]),
        "properties_written": set([]),
    },
    "Pitched_trill_engraver": {
        "grobs_created": set([
            "TrillPitchAccidental",
            "TrillPitchGroup",
            "TrillPitchHead",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Pure_from_neighbor_engraver": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Repeat_acknowledge_engraver": {
        "grobs_created": set([]),
        "properties_read": set([
            "doubleRepeatSegnoType",
            "doubleRepeatType",
            "endRepeatSegnoType",
            "endRepeatType",
            "repeatCommands",
            "segnoType",
            "startRepeatSegnoType",
            "startRepeatType",
            "whichBar",
            ]),
        "properties_written": set([]),
    },
    "Repeat_tie_engraver": {
        "grobs_created": set([
            "RepeatTie",
            "RepeatTieColumn",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Rest_collision_engraver": {
        "grobs_created": set([
            "RestCollision",
            ]),
        "properties_read": set([
            "busyGrobs",
            ]),
        "properties_written": set([]),
    },
    "Rest_engraver": {
        "grobs_created": set([
            "Rest",
            ]),
        "properties_read": set([
            "middleCPosition",
            ]),
        "properties_written": set([]),
    },
    "Rhythmic_column_engraver": {
        "grobs_created": set([
            "NoteColumn",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Scheme_engraver": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Script_column_engraver": {
        "grobs_created": set([
            "ScriptColumn",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Script_engraver": {
        "grobs_created": set([
            "Script",
            ]),
        "properties_read": set([
            "scriptDefinitions",
            ]),
        "properties_written": set([]),
    },
    "Script_row_engraver": {
        "grobs_created": set([
            "ScriptRow",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Separating_line_group_engraver": {
        "grobs_created": set([
            "StaffSpacing",
            ]),
        "properties_read": set([
            "createSpacing",
            ]),
        "properties_written": set([
            "hasStaffSpacing",
            ]),
    },
    "Slash_repeat_engraver": {
        "grobs_created": set([
            "DoubleRepeatSlash",
            "RepeatSlash",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Slur_engraver": {
        "grobs_created": set([
            "Slur",
            ]),
        "properties_read": set([
            "doubleSlurs",
            "slurMelismaBusy",
            ]),
        "properties_written": set([]),
    },
    "Slur_performer": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Spacing_engraver": {
        "grobs_created": set([
            "SpacingSpanner",
            ]),
        "properties_read": set([
            "currentCommandColumn",
            "currentMusicalColumn",
            "proportionalNotationDuration",
            ]),
        "properties_written": set([]),
    },
    "Span_arpeggio_engraver": {
        "grobs_created": set([
            "Arpeggio",
            ]),
        "properties_read": set([
            "connectArpeggios",
            ]),
        "properties_written": set([]),
    },
    "Span_bar_engraver": {
        "grobs_created": set([
            "SpanBar",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Span_bar_stub_engraver": {
        "grobs_created": set([
            "SpanBarStub",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Spanner_break_forbid_engraver": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Staff_collecting_engraver": {
        "grobs_created": set([]),
        "properties_read": set([
            "stavesFound",
            ]),
        "properties_written": set([
            "stavesFound",
            ]),
    },
    "Staff_performer": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Staff_symbol_engraver": {
        "grobs_created": set([
            "StaffSymbol",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Stanza_number_align_engraver": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Stanza_number_engraver": {
        "grobs_created": set([
            "StanzaNumber",
            ]),
        "properties_read": set([
            "stanza",
            ]),
        "properties_written": set([]),
    },
    "Stem_engraver": {
        "grobs_created": set([
            "Flag",
            "Stem",
            "StemStub",
            "StemTremolo",
            ]),
        "properties_read": set([
            "stemLeftBeamCount",
            "stemRightBeamCount",
            "whichBar",
            ]),
        "properties_written": set([]),
    },
    "System_start_delimiter_engraver": {
        "grobs_created": set([
            "SystemStartBar",
            "SystemStartBrace",
            "SystemStartBracket",
            "SystemStartSquare",
            ]),
        "properties_read": set([
            "currentCommandColumn",
            "systemStartDelimiter",
            "systemStartDelimiterHierarchy",
            ]),
        "properties_written": set([]),
    },
    "Tab_note_heads_engraver": {
        "grobs_created": set([
            "TabNoteHead",
            ]),
        "properties_read": set([
            "defaultStrings",
            "fretLabels",
            "highStringOne",
            "middleCPosition",
            "minimumFret",
            "noteToFretFunction",
            "stringOneTopmost",
            "stringTunings",
            "tabStaffLineLayoutFunction",
            "tablatureFormat",
            ]),
        "properties_written": set([]),
    },
    "Tab_staff_symbol_engraver": {
        "grobs_created": set([
            "StaffSymbol",
            ]),
        "properties_read": set([
            "stringTunings",
            ]),
        "properties_written": set([]),
    },
    "Tab_tie_follow_engraver": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Tempo_performer": {
        "grobs_created": set([]),
        "properties_read": set([
            "tempoWholesPerMinute",
            ]),
        "properties_written": set([]),
    },
    "Text_engraver": {
        "grobs_created": set([
            "TextScript",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Text_spanner_engraver": {
        "grobs_created": set([
            "TextSpanner",
            ]),
        "properties_read": set([
            "currentMusicalColumn",
            ]),
        "properties_written": set([]),
    },
    "Tie_engraver": {
        "grobs_created": set([
            "Tie",
            "TieColumn",
            ]),
        "properties_read": set([
            "skipTypesetting",
            "tieWaitForNote",
            ]),
        "properties_written": set([
            "tieMelismaBusy",
            ]),
    },
    "Tie_performer": {
        "grobs_created": set([]),
        "properties_read": set([
            "tieWaitForNote",
            ]),
        "properties_written": set([
            "tieMelismaBusy",
            ]),
    },
    "Time_signature_engraver": {
        "grobs_created": set([
            "TimeSignature",
            ]),
        "properties_read": set([
            "initialTimeSignatureVisibility",
            "partialBusy",
            "timeSignatureFraction",
            ]),
        "properties_written": set([]),
    },
    "Time_signature_performer": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Timing_translator": {
        "grobs_created": set([]),
        "properties_read": set([
            "baseMoment",
            "currentBarNumber",
            "internalBarNumber",
            "measureLength",
            "measurePosition",
            "timeSignatureFraction",
            ]),
        "properties_written": set([
            "baseMoment",
            "currentBarNumber",
            "internalBarNumber",
            "measureLength",
            "measurePosition",
            "timeSignatureFraction",
            ]),
    },
    "Translator": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Trill_spanner_engraver": {
        "grobs_created": set([
            "TrillSpanner",
            ]),
        "properties_read": set([
            "currentCommandColumn",
            "currentMusicalColumn",
            ]),
        "properties_written": set([]),
    },
    "Tuplet_engraver": {
        "grobs_created": set([
            "TupletBracket",
            "TupletNumber",
            ]),
        "properties_read": set([
            "tupletFullLength",
            "tupletFullLengthNote",
            ]),
        "properties_written": set([]),
    },
    "Tweak_engraver": {
        "grobs_created": set([]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Vaticana_ligature_engraver": {
        "grobs_created": set([
            "DotColumn",
            "VaticanaLigature",
            ]),
        "properties_read": set([]),
        "properties_written": set([]),
    },
    "Vertical_align_engraver": {
        "grobs_created": set([
            "VerticalAlignment",
            ]),
        "properties_read": set([
            "alignAboveContext",
            "alignBelowContext",
            "hasAxisGroup",
            ]),
        "properties_written": set([]),
    },
    "Volta_engraver": {
        "grobs_created": set([
            "VoltaBracket",
            "VoltaBracketSpanner",
            ]),
        "properties_read": set([
            "repeatCommands",
            "stavesFound",
            "voltaSpannerDuration",
            ]),
        "properties_written": set([]),
    },
    }



import collections
from abjad.system.AbjadValueObject import AbjadValueObject


class PitchInequality(AbjadValueObject):
    """
    Pitch inequality.

    ..  container:: example

        >>> inequality = abjad.PitchInequality('&', 'C4 E4')
        >>> abjad.f(inequality)
        abjad.PitchInequality(
            operator_string='&',
            pitches=abjad.PitchSet(
                [0, 4]
                ),
            )

        >>> inequality(abjad.Staff("d'8 e' f' g'"))
        True

        >>> inequality(abjad.Staff("e'8 f' g' a'"))
        True

        >>> inequality(abjad.Staff("f'8 g' a' b'"))
        False

    .. note:: only intersection currently implemented.

    """

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Inequalities'

    __slots__ = (
        '_operator_string',
        '_pitches',
        )

    _set_theoretic_operator_strings = (
        '&',
        '|',
        '^',
        )

    _publish_storage_format = True

    ### INITIALIZER ###

    def __init__(
        self,
        operator_string='&',
        pitches=None,
        ):
        import abjad
        assert operator_string in self._set_theoretic_operator_strings
        self._operator_string = operator_string
        # only intersection is currently implemented
        if not isinstance(pitches, collections.Iterable):
            pitches = [pitches]
        pitches = abjad.PitchSet(
            items=pitches,
            item_class=abjad.NumberedPitch,
            )
        self._pitches = pitches

    ### SPECIAL METHODS ###

    def __call__(self, argument):
        """
        Calls inequality on ``argument``.

        Returns true or false.
        """
        import abjad
        if not self.pitches:
            return False
        pitch_set = abjad.PitchSet.from_selection(
            argument,
            item_class=abjad.NumberedPitch,
            )
        if self.operator_string == '&':
            return bool(self.pitches.intersection(pitch_set))
        else:
            message = 'implement {!r}.'
            message = message.format(self.operator_string)
            raise NotImplementedError(message)

    ### PUBLIC PROPERTIES ###

    @property
    def operator_string(self):
        """
        Gets operator string.

        Returns string.
        """
        return self._operator_string

    @property
    def pitches(self):
        """
        Gets pitches.

        Returns numbered pitch set.
        """
        return self._pitches



def list_all_classes(modules=None, ignored_classes=None):
    r'''Lists all public classes defined in `path`.

    ..  container:: example

        ::

            >>> all_classes = abjad.utilities.list_all_classes(
            ...     modules='abjad',
            ...     )

    '''
    from abjad import utilities
    all_classes = set()
    for module in utilities.yield_all_modules(modules):
        name = module.__name__.split('.')[-1]
        if name.startswith('_'):
            continue
        if not hasattr(module, name):
            continue
        obj = getattr(module, name)
        if isinstance(obj, type):
            all_classes.add(obj)
    if ignored_classes:
        ignored_classes = set(ignored_classes)
        all_classes.difference_update(ignored_classes)
    return list(sorted(
        all_classes,
        key=lambda x: (x.__module__, x.__name__)
        ))



import collections
import numbers


def are_relatively_prime(argument):
    """
    Is true when ``argument`` is an iterable collection of relative primes.

    ..  container:: example

        >>> abjad.mathtools.are_relatively_prime([13, 14, 15])
        True

        >>> abjad.mathtools.are_relatively_prime([13, 14, 15, 16])
        False

        >>> abjad.mathtools.are_relatively_prime('text')
        False

    ..  container:: example

        Returns true when ``argument`` is empty:

        >>> abjad.mathtools.are_relatively_prime([])
        True

    Returns true or false.
    """
    from abjad import mathtools
    if not isinstance(argument, collections.Iterable):
        return False
    if not all(isinstance(_, numbers.Number) for _ in argument):
        return False
    all_factors = set([])
    for number in argument:
        current_factors = mathtools.factors(number)
        current_factors = set(current_factors)
        if all_factors & current_factors:
            return False
        all_factors.update(current_factors)
    return True



import typing
from abjad.system.AbjadValueObject import AbjadValueObject
from abjad.system.FormatSpecification import FormatSpecification
from abjad.system.LilyPondFormatBundle import LilyPondFormatBundle


class BarLine(AbjadValueObject):
    r"""
    Bar line.

    ..  container:: example

        Final bar line:

        >>> staff = abjad.Staff("c'4 d'4 e'4 f'4")
        >>> bar_line = abjad.BarLine('|.')
        >>> abjad.attach(bar_line, staff[-1])
        >>> abjad.show(staff) # doctest: +SKIP

        >>> bar_line
        BarLine('|.')

        ..  docs::

            >>> abjad.f(staff)
            \new Staff
            {
                c'4
                d'4
                e'4
                f'4
                \bar "|."
            }

    """

    ### CLASS VARIABLES ###

    __slots__ = (
        '_abbreviation',
        )

    _context = 'Staff'

    _format_slot = 'closing'

    ### INITIALIZER ##

    def __init__(self, abbreviation: str = '|') -> None:
        assert isinstance(abbreviation, str), repr(abbreviation)
        self._abbreviation = abbreviation

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        return FormatSpecification(
            client=self,
            storage_format_is_indented=False,
            storage_format_args_values=[self.abbreviation],
            )

    def _get_lilypond_format(self):
        return rf'\bar "{self.abbreviation}"'

    def _get_lilypond_format_bundle(self, component=None):
        bundle = LilyPondFormatBundle()
        bundle.after.commands.append(self._get_lilypond_format())
        return bundle

    ## PUBLIC PROPERTIES ##

    @property
    def abbreviation(self) -> str:
        r"""
        Gets abbreviation.

        ..  container:: example

            >>> bar_line = abjad.BarLine('|.')
            >>> bar_line.abbreviation
            '|.'

        """
        return self._abbreviation

    @property
    def context(self) -> str:
        r"""
        Gets (historically conventional) context.

        ..  container:: example

            >>> bar_line = abjad.BarLine('|.')
            >>> bar_line.context
            'Staff'

        ..  todo:: Should return ``'Score'``.

        Override with ``abjad.attach(..., context='...')``.
        """
        return self._context

    @property
    def tweaks(self) -> None:
        r"""
        Are not implemented on bar line.
        
        The LilyPond ``\bar`` command refuses tweaks.

        Use overrides instead.
        """
        pass



def timespan_2_starts_during_timespan_1(
    timespan_1=None,
    timespan_2=None,
    hold=False,
    ):
    """
    Makes time relation indicating that ``timespan_2`` starts
    during ``timespan_1``.

    ..  container:: example

        >>> relation = abjad.timespans.timespan_2_starts_during_timespan_1()
        >>> abjad.f(relation)
        abjad.timespans.TimespanTimespanTimeRelation(
            inequality=abjad.timespans.CompoundInequality(
                [
                    abjad.TimespanInequality('timespan_1.start_offset <= timespan_2.start_offset'),
                    abjad.TimespanInequality('timespan_2.start_offset < timespan_1.stop_offset'),
                    ],
                logical_operator='and',
                ),
            )

    ..  container:: example

        >>> staff_1 = abjad.Staff("c'4 d'4 e'4 f'4 g'2 c''2")
        >>> staff_2 = abjad.Staff("c'2 b'2 a'2 g'2")
        >>> score = abjad.Score([staff_1, staff_2])
        >>> abjad.show(score) # doctest: +SKIP

        >>> start_offsets = [
        ...     abjad.inspect(note).timespan().start_offset
        ...     for note in staff_1
        ...     ]
        >>> stop_offsets = [
        ...     abjad.inspect(note).timespan().stop_offset
        ...     for note in staff_1
        ...     ]

        >>> timespan_1 = abjad.Timespan((1, 4), (5, 4))
        >>> time_relation = abjad.timespans.timespan_2_starts_during_timespan_1(
        ...     timespan_1=timespan_1)
        >>> start_index, stop_index = time_relation.get_offset_indices(
        ...     start_offsets, stop_offsets)

        >>> selected_notes = staff_1[start_index:stop_index]
        >>> selected_notes
        Selection([Note("d'4"), Note("e'4"), Note("f'4"), Note("g'2")])

        >>> abjad.label(selected_notes).color_leaves('red')

        >>> abjad.show(score) # doctest: +SKIP

    Returns time relation or boolean.
    """
    from abjad import timespans

    inequality = timespans.CompoundInequality([
        'timespan_1.start_offset <= timespan_2.start_offset',
        'timespan_2.start_offset < timespan_1.stop_offset',
        ])

    time_relation = timespans.TimespanTimespanTimeRelation(
        inequality,
        timespan_1=timespan_1,
        timespan_2=timespan_2,
        )

    if time_relation.is_fully_loaded and not hold:
        return time_relation()
    else:
        return time_relation
